unit uFunc;

interface

uses
  Classes, Windows, Dialogs, IniFiles, uConst, Generics.Collections;

type
  TCommFortProcess = procedure(dwPluginID : DWORD; dID: DWORD; bOutBuffer : PAnsiChar; dwOutBufferSize : DWORD); stdcall;
  TCommFortGetData = function(dwPluginID : DWORD; dwID : DWORD; bInBuffer : PAnsiChar; dwInBufferSize : DWORD; bOutBuffer : PAnsiChar; dwOutBufferSize : DWORD): DWORD; stdcall;

  TPaintDataEntry = record
    IsPicture : Boolean;
    PaintData : String;
    PaintDataByteCount : Integer;
    PaintDataPixelCount : Integer;
  end;

  TOptions = record
    BotName : String;
    BotPassword : String;
    BotIP : String;
    BotIsMale : Boolean;
    PaintChannel : String;
    SavePictures : Boolean;
  end;

  function  fReadInteger(bInBuffer : PAnsiChar; var iOffset : Integer): Integer;
  function  fReadText(bInBuffer : PAnsiChar; var iOffset : Integer): WideString;
  procedure fWriteInteger(var bOutBuffer : PAnsiChar; var iOffset  : Integer; iValue : Integer);
  procedure fWriteText(bOutBuffer : PAnsiChar; var iOffset  : Integer; uValue : WideString);
  function  fTextToAnsiString(uText : WideString) : AnsiString;
  function  fIntegerToAnsiString(iValue : Integer) : AnsiString;

  procedure CreateVirtualUser(const AVUserName, AIP, APassword : String; AMale : Boolean = True);
  procedure DisconnectVirtualUser(const AVUserName : String);
  procedure MsgToChannel(const AVUserName, AChannel, AText : String; const ANormalMessage : Boolean = True);
  procedure MsgToPrivatChannel(const AVUserName, ADestUserName, AText : String; const ANormalMessage : Boolean = True);
  procedure ImageToChannel(const AVUserName, AChannel, JpegData : String);
  procedure CreateOrEnterChannel(const AVUserName, AChannel: String; const AVisible : Boolean = True; const APrivate : Boolean = False);
  procedure LeaveChannel(const AVUserName, AChannel: String);
  procedure ChangeChannelTopic(const AVUserName, AChannel : String; ATopic : String);
  function GetFullSettingsFileName(): string;
  procedure LoadOptionsFromFile(var AOptions : TOptions);
  procedure SaveOptionsToFile(AOptions : TOptions);
  function ExtractPaintDataString(var AData : String): Boolean;
  procedure ClearPaintDataIfNeed(APaintDataEntryList : TList<TPaintDataEntry>);

  function GetImagePixelCount(APImageStr : PAnsiChar; AImageStrSize : Integer) : Integer;
//  procedure AddLogStr(AFunctionName, AText : String; const AError : Boolean = True);

  function GetImageLimitation(AChanName : String): TSize;
var
  dwPluginID : DWORD;
  CommFortProcess : TCommFortProcess;
  CommFortGetData : TCommFortGetData;
  Options : TOptions;

implementation
uses StrUtils, SysUtils;
//---------------------------------------------------------------------------
function fReadInteger(bInBuffer : PAnsiChar; var iOffset : Integer): Integer; //       
begin
	CopyMemory(@Result, bInBuffer + iOffSet, 4);
	iOffset := iOffset + 4;
end;

function fReadText(bInBuffer : PAnsiChar; var iOffset : Integer): WideString; //       
 var iLength : Integer;
begin
	CopyMemory(@iLength, bInBuffer + iOffSet, 4);
	iOffset := iOffset + 4;
	SetLength(Result, iLength);
	CopyMemory(@Result[1], bInBuffer + iOffSet, iLength * 2);
	iOffset := iOffset + iLength * 2;
end;

//---------------------------------------------------------------------------
procedure fWriteInteger(var bOutBuffer : PAnsiChar; var iOffset  : Integer; iValue : Integer); //       
begin
	CopyMemory(bOutBuffer + iOffSet, @iValue, 4);
	iOffset := iOffset + 4;
end;
//---------------------------------------------------------------------------
procedure fWriteText(bOutBuffer : PAnsiChar; var iOffset  : Integer; uValue : WideString); //       
	var iLength : Integer;
begin
	iLength := Length(uValue);
	CopyMemory(bOutBuffer + iOffset, @iLength, 4);
	iOffset := iOffset + 4;
	CopyMemory(bOutBuffer + iOffSet, @uValue[1], iLength * 2);
	iOffset := iLength * 2;
end;

//---------------------------------------------------------------------------
function fTextToAnsiString(uText : WideString) : AnsiString; //      
	var iLength : Integer;
begin
	iLength := Length(uText);

	SetLength(Result, 4 + iLength * 2);

	CopyMemory(@Result[1], @iLength, 4);
	CopyMemory(PAnsiChar(Result) + 4, @uText[1], iLength * 2);
end;
//---------------------------------------------------------------------------
function fIntegerToAnsiString(iValue : Integer) : AnsiString; //      
begin
	SetLength(Result, 4);
	CopyMemory(@Result[1], @iValue, 4);
end;

procedure CreateVirtualUser(const AVUserName, AIP, APassword : String; AMale : Boolean = True);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AVUserName)+
	 fTextToAnsiString(AIP)+
	 fIntegerToAnsiString(0)+  //   
	 fTextToAnsiString(APassword)+
	 fIntegerToAnsiString(Integer(not AMale));
	CommFortProcess(dwPluginID, 1001, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure DisconnectVirtualUser(const AVUserName : String);
var aDataToSend : AnsiString;
begin
  if Length(AVUserName) > 0 then
    begin
      aDataToSend := fTextToAnsiString(AVUserName);
      CommFortProcess(dwPluginID, 1002, PAnsiChar(aDataToSend), Length(aDataToSend));
    end;
end;

procedure CreateOrEnterChannel(const AVUserName, AChannel: String; const AVisible : Boolean = True; const APrivate : Boolean = False);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AVUserName)+
   fTextToAnsiString(AChannel)+
   fIntegerToAnsiString(Integer(AVisible))+
   fIntegerToAnsiString(Integer(APrivate));
	CommFortProcess(dwPluginID, 1026, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure LeaveChannel(const AVUserName, AChannel: String);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AVUserName)+
   fTextToAnsiString(AChannel);
	CommFortProcess(dwPluginID, 1027, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure MsgToChannel(const AVUserName, AChannel, AText : String; const ANormalMessage : Boolean = True);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AVUserName)+
	 fIntegerToAnsiString(Integer(not ANormalMessage))+
	 fTextToAnsiString(AChannel)+
 	 fTextToAnsiString(AText);
	CommFortProcess(dwPluginID, 1020, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure MsgToPrivatChannel(const AVUserName, ADestUserName, AText : String; const ANormalMessage : Boolean = True);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AVUserName)+
	 fIntegerToAnsiString(Integer(not ANormalMessage))+
	 fTextToAnsiString(ADestUserName)+
 	 fTextToAnsiString(AText);
	CommFortProcess(dwPluginID, 1021, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure ImageToChannel(const AVUserName, AChannel, JpegData : String);
var aDataToSend : AnsiString;
    iSendCode : Integer;
begin
  aDataToSend :=
    fTextToAnsiString(AVUserName) +
    fTextToAnsiString(AChannel) +
    fTextToAnsiString(JpegData);
  CommFortProcess(dwPluginID, iSendCode, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure ChangeChannelTopic(const AVUserName, AChannel : String; ATopic : String);
var aDataToSend : AnsiString;
begin
  if Length(ATopic) > 255 then
    SetLength(ATopic, 255);
	aDataToSend := fTextToAnsiString(AVUserName)+
	 fTextToAnsiString(AChannel)+
 	 fTextToAnsiString(ATopic);
	CommFortProcess(dwPluginID, 1023, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

function GetImageLimitation(AChanName : String): TSize;
var iSize : Integer;
    aData : AnsiString;
    PaData : PAnsiChar;

    outData : AnsiString;
    iReadOffset : Integer;
begin
  outData := fTextToAnsiString(AChanName);
	iSize := CommFortGetData(dwPluginID, 1030, nil, 0, PAnsiChar(outData), Length(outData));
  if iSize > 0 then
    begin
      SetLength(aData, iSize);
      PaData := PAnsiChar(aData);

      iSize := CommFortGetData(dwPluginID, 1030, PaData, iSize, PAnsiChar(outData), Length(outData));
      SetLength(aData, iSize);
      iReadOffset := 0;
      Result.cx := fReadInteger(PaData, iReadOffset);
      Result.cy := fReadInteger(PaData, iReadOffset);
    end;
end;

function GetImagePixelCount(APImageStr : PAnsiChar; AImageStrSize : Integer) : Integer;
var PEndOfImageStr : PAnsiChar;
    wSize : WORD;
begin
  Result := 0;
  PEndOfImageStr := APImageStr + AImageStrSize;
  while APImageStr < (PEndOfImageStr - $20) do
  begin
    if (APImageStr^ = #$FF) and ((APImageStr + 1)^ = #$C0) then
    begin
      Inc(APImageStr, 5);
      CopyMemory(@wSize, APImageStr, 2);
      Result := wSize;
      Inc(APImageStr, 5);
      CopyMemory(@wSize, APImageStr, 2);
      Result := Result * wSize;
      Exit;
    end;
    Inc(APImageStr);
  end;
end;


function GetFullSettingsFileName(): string;
var iSize : Integer;
    aData : AnsiString;
    PaData : PAnsiChar;
    iReadOffset : Integer;
begin
  SetLength(Result, 0);
	iSize := CommFortGetData(dwPluginID, 2010, nil, 0, nil, 0);
  if iSize > 0 then
    begin
      SetLength(aData, iSize);
      PaData := PAnsiChar(aData);
      iSize := CommFortGetData(dwPluginID, 2010, PaData, iSize, nil, 0);
      SetLength(aData, iSize);
      iReadOffset := 0;
      Result := fReadText(PaData, iReadOffset);
    end;
  try
    ForceDirectories(Result);
  except
     on E : Exception do
       ShowMessage(E.Message);
  end;
  Result := IncludeTrailingPathDelimiter(Result) + INIFILE;
end;

procedure LoadOptionsFromFile(var AOptions : TOptions);
var IniFile : TIniFile;
  function GetRandomString(NumChar: cardinal): string;
  const
    CharMap='qwertzuiopasdfghjklyxcvbnmQWERTZUIOPASDFGHJKLYXCVBNM1234567890';
  var
    i : Integer;
    MaxChar : Cardinal;
  begin
    Randomize;
    MaxChar := Length(CharMap) - 1;
    SetLength(Result, NumChar);
    for i := 1 to NumChar do
    begin
      Result[i] := CharMap[Random(MaxChar) + 1];
    end;
  end;
begin
  try
    IniFile := TIniFile.Create(GetFullSettingsFileName());
    try
      AOptions.PaintChannel := IniFile.ReadString('General', 'PaintChannel', 'paintboard');
      AOptions.BotName := IniFile.ReadString('General', 'BotName', 'PaintBot');
      AOptions.BotPassword := IniFile.ReadString('General', 'BotPassword', '');
      AOptions.BotIP := IniFile.ReadString('General', 'BotIP', '8.8.8.8');
      AOptions.BotIsMale := IniFile.ReadBool('General', 'BotIsMale', True);
      AOptions.SavePictures := IniFile.ReadBool('General', 'SavePictures', True);
      if Length(AOptions.BotPassword) = 0 then
        begin
          AOptions.BotPassword := GetRandomString(12);
          SaveOptionsToFile(AOptions);
        end;
    finally
       FreeAndNil(IniFile);
    end;
  except
     on E : Exception do
       ShowMessage('LoadOptionsFromFile() ' + E.Message);
  end;
end;

procedure SaveOptionsToFile(AOptions : TOptions);
var IniFile : TIniFile;
begin
  try
    IniFile := TIniFile.Create(GetFullSettingsFileName());
    try
      IniFile.WriteString('General', 'BotName', AOptions.BotName);
      IniFile.WriteString('General', 'BotPassword', AOptions.BotPassword);
      IniFile.WriteString('General', 'PaintChannel', AOptions.PaintChannel);
      IniFile.WriteString('General', 'BotIP', AOptions.BotIP);
      IniFile.WriteBool('General', 'BotIsMale', AOptions.BotIsMale);
      IniFile.WriteBool('General', 'SavePictures', AOptions.SavePictures);
    finally
       FreeAndNil(IniFile);
    end;
  except
     on E : Exception do
       ShowMessage('SaveOptionsToFile() ' + E.Message);
  end;
end;

function ExtractPaintDataString(var AData : String): Boolean;
begin
  Result := StartsText(START_PAINT_MESSAGE, AData) and
    EndsText(END_PAINT_MESSAGE, AData);
  if Result then
    begin
      Delete(AData, 1, Length(START_PAINT_MESSAGE));
      SetLength(AData, Length(AData) - Length(END_PAINT_MESSAGE));
    end;
end;

procedure ClearPaintDataIfNeed(APaintDataEntryList : TList<TPaintDataEntry>);
var i : Integer;
    TmpStr : String;
    iCounter : Integer;
    bNeedClear : Boolean;
    pdeCurrent : TPaintDataEntry;
begin
  iCounter := 0;
  bNeedClear := False;

  for i := APaintDataEntryList.Count - 1 downto 0 do
  begin
    if bNeedClear then
    begin
      APaintDataEntryList.Delete(i);
      Continue;
    end;
    pdeCurrent := APaintDataEntryList[i];
    if (not pdeCurrent.IsPicture) and (pdeCurrent.PaintData = 'CLBRD') then
    begin
      if iCounter >= MAX_UNDO_COUNT - 1 then
      begin
        bNeedClear := True;
        APaintDataEntryList.Delete(i);
      end
      else
        Inc(iCounter);
    end
    else
    begin
      if pdeCurrent.PaintData = 'UNDO' then
        Dec(iCounter)
      else
        Inc(iCounter);
    end;
  end;
  i := 0;
  while (i <= APaintDataEntryList.Count - 2) do
  begin
    if (i > 0) and
       (APaintDataEntryList[i].PaintData = 'UNDO') and
       (APaintDataEntryList[i + 1].PaintData <> 'REDO') and
       (APaintDataEntryList[i + 1].PaintData <> 'UNDO') then
    begin
      APaintDataEntryList.Delete(i - 1);
      APaintDataEntryList.Delete(i - 1);
    end
    else
    if (APaintDataEntryList[i].PaintData = 'UNDO') and
       (APaintDataEntryList[i + 1].PaintData = 'REDO') then
    begin
      APaintDataEntryList.Delete(i);
      APaintDataEntryList.Delete(i);
      Dec(i);
    end
    else
      Inc(i);
  end;
end;

//procedure AddLogStr(AFunctionName, AText : String; const AError : Boolean = True);
//var aDataToSend : AnsiString;
//begin
//	aDataToSend := fIntegerToAnsiString(Integer(AError) * 2) +
//	 fIntegerToAnsiString(0) +
//	 fIntegerToAnsiString(0) +
// 	 fTextToAnsiString('PaintBoard ' + AFunctionName + ' : ' + AText);
//	CommFortProcess(dwPluginID, 100, PAnsiChar(aDataToSend), Length(aDataToSend));
//end;



end.
