unit uCommonGraphics;

interface

uses
  Windows, SysUtils, Graphics, Classes, GR32_Image, Types, GR32, GR32_Blend, GR32_Filters, Math,
  Controls, acAlphaImageList, G32_InterFace, GR32_Layers, sSkinManager, ImgList;

type TFigureFilling = (ffFill, ffBorder, ffBorderAndFill);
type
  TTexture = class
  private
    FIsColor: Boolean;
    FTextureColor: TColor32;
    FTextureBitmap: TBitmap32;
    FTextureBitmapIndex: Integer;
    procedure SetIsColor(const Value: Boolean);
    procedure SetTextureColor(const Value: TColor32);
    procedure SetTextureBitmap(const Value: TBitmap32);
    procedure SetTextureBitmapIndex(const Value: Integer);
    { Private declarations }
  public
    { Public declarations }
    property IsColor : Boolean read FIsColor write SetIsColor;
    property TextureColor : TColor32 read FTextureColor write SetTextureColor;
    property TextureBitmap  : TBitmap32 read FTextureBitmap write SetTextureBitmap;
    property TextureBitmapIndex : Integer read FTextureBitmapIndex write SetTextureBitmapIndex;

    function EncodeTexture() : String;
    procedure DecodeTexture(AInputString : String);
    constructor Create();
    destructor Destroy(); override;
  end;

  TMyBitmap32 = class(TBitmap32)
  private
    FTag: Integer;
    procedure SetTag(const Value: Integer);
  published
  public
    property Tag : Integer read FTag write SetTag;
  end;

type
  TdmCommonGraphics = class(TDataModule)
    aiBrushes: TsAlphaImageList;
    aiTextures: TsAlphaImageList;
    lPen: TBitmap32List;
    aiStampes: TsAlphaImageList;
    sSkinManagerPB: TsSkinManager;
    aiButtons24: TsAlphaImageList;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    FDrawImage : TImage32;
    FDrawImageLayerBitmap : TBitmap32;
    FTextureList : TList;
    FStampList : TList;
    FBrushesList : TList;
//    procedure PC_Modulate(F: TColor32; var B: TColor32; M: TColor32);
    function GetBitmapFromListOrRes(AList : TList; AIndex : Integer; AResPrefix : String) : TBitmap32;
  public
    procedure TileTexure(ADest : TBitmap32; ATexture: TTexture);
    procedure DrawPolyLine(APointArray: TArrayOfPoint; ADest, ABrush, AAlphaBitmap : TBitmap32;
      ASegmentsToDrawCount : Integer; ATexture : TTexture);

    procedure DrawLine(AX1, AY1, AX2, AY2 : Integer; ADest, ABrush, AAlphaBitmap : TBitmap32; ATexture : TTexture);

    procedure DrawStamp(AX, AY, AStampSize : Integer; ADest, AStamp, AAlphaBitmap : TBitmap32; ATexture : TTexture);

    procedure FloodFill(AX, AY, ATolerance : Integer; ASourceImage : TImage32; ADest : TBitmap32;
      ATexture : TTexture);
    procedure FloodFill1(AX, AY, ATolerance : Integer; ASourceImage : TImage32; ADest : TBitmap32;
      ATexture : TTexture);

    procedure DrawFigure(AX1, AY1, AX2, AY2 : Integer; ADest, AAlphaBitmap : TBitmap32;
      ADrawEllipse : Boolean; APenWidth : Integer; AFigureFilling : TFigureFilling; APenTexture : TTexture; ABrushTexture : TTexture);

    procedure DrawText(AX, AY : Integer; ADest, AAlphaBitmap : TBitmap32; AText : string; ATexture : TTexture; ADrawCursor : Boolean = False);

    procedure Resize(ADest : TBitmap32; ANewSize : Integer);
    procedure DrawGraphicToBitmap(ADest : TBitmap32; ASource : TGraphic; ADestRect : TRect);

    function GetTexture(AIndex : Integer) : TBitmap32;
    function GetStamp(AIndex : Integer) : TBitmap32;
    function GetBrush(AIndex : Integer) : TBitmap32;
    //    procedure CustomDraw(ABackground, AForeground : TBitmap32);
  end;

var
  dmCommonGraphics: TdmCommonGraphics = nil;

implementation

uses uFunc, GR32_Resamplers;

{$R *.dfm}

//procedure TdmCommonGraphics.CustomDraw(ABackground, AForeground: TBitmap32);
//var i, j : Integer;
//    F, B : TColor32;
//    FX : TColor32Entry absolute F;
//    BX : TColor32Entry absolute B;
//begin
//  for i := 0 to ABackground.Width - 1 do
//    for j := 0 to ABackground.Height - 1 do
//      begin
//        B := ABackground.PixelS[i, j];
//        F := AForeground.PixelS[i, j];
//        if FX.A = $FF then
//          B := F
//        else
//        if FX.A = 0 then
//          Continue
//        else
//          begin
//            try
//
//              B := BlendReg(F, B); //BX.A := BX.A + ((BX.A div 256) * FX.A);
//              BX.A := 255;
//            except
//              BlendReg(F, B);
//              B := F;
//            end;
//          end;
//        ABackground.PixelS[i, j] := B;
//      end;
//end;

procedure TdmCommonGraphics.DataModuleCreate(Sender: TObject);
var ALayer : TBitmapLayer;
    L : TFloatRect;
    i : Integer;
    ResName : string;

    procedure LoadIconsFromResource(AResFormatStr : String; AImageList : TsAlphaImageList);
    var i : Integer;
    begin
      i := 0;
      while True do
      begin
        ResName := Format(AResFormatStr, [i]);
        if FindResource(HInstance, PChar(ResName), RT_RCDATA) <> 0 then
          AddImageFromRes(Hinstance, AImageList, ResName, ifPNG)
        else
          Break;
        Inc(i);
      end;
    end;
begin
  inherited;
  try
    FDrawImage := TImage32.Create(nil);
    FDrawImage.Width := 800;
    FDrawImage.Height := 600;
    FDrawImage.BeginUpdate();
    try
      ALayer := TBitmapLayer.Create(FDrawImage.Layers);
      with ALayer do
      begin
        Bitmap := TBitmap32.Create();
        Bitmap.DrawMode := dmBlend;
        Bitmap.SetSizeFrom(FDrawImage);
        Bitmap.MasterAlpha := 255;
        Bitmap.Clear(clBlack);
        L.Left := 0;
        L.Top := 0;
        L.Right := Bitmap.Width;
        L.Bottom := Bitmap.Height;
        ALayer.Location := L;
        FDrawImageLayerBitmap := Bitmap;
      end;
      FDrawImage.Bitmap := TBitmap32.Create();
      FDrawImage.Bitmap.SetSizeFrom(FDrawImage);
      FDrawImage.Bitmap.Clear(clWhite32);
    finally
      FDrawImage.EndUpdate;
      FDrawImage.Changed;
    end;

    LoadIconsFromResource('T%.3dI', aiTextures);
    LoadIconsFromResource('S%.3dI', aiStampes);
    LoadIconsFromResource('B%.3dI', aiBrushes);

    FTextureList := TList.Create();
    for i := 0 to aiTextures.Count - 1 do
      FTextureList.Add(nil);
    FStampList := TList.Create();
    for i := 0 to aiStampes.Count - 1 do
      FStampList.Add(nil);
    FBrushesList := TList.Create();
    for i := 0 to aiBrushes.Count - 1 do
      FBrushesList.Add(nil);

  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.DataModuleCreate()');
  end;
end;

procedure TdmCommonGraphics.DataModuleDestroy(Sender: TObject);
var i : Integer;
begin
  try
    FreeAndNil(FDrawImage);
    try
      for i := 0 to FTextureList.Count - 1 do
      begin
        if Assigned(FTextureList[i]) then
          TBitmap32(FTextureList[i]).Free();
      end;
    finally
      FreeAndNil(FTextureList);
    end;
    try
      for i := 0 to FStampList.Count - 1 do
      begin
        if Assigned(FStampList[i]) then
          TBitmap32(FStampList[i]).Free();
      end;
    finally
      FreeAndNil(FStampList);
    end;
    try
      for i := 0 to FBrushesList.Count - 1 do
      begin
        if Assigned(FBrushesList[i]) then
          TBitmap32(FBrushesList[i]).Free();
      end;
    finally
      FreeAndNil(FBrushesList);
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.DataModuleDestroy()');
  end;
end;

procedure TdmCommonGraphics.DrawFigure(AX1, AY1, AX2, AY2 : Integer; ADest, AAlphaBitmap : TBitmap32;
      ADrawEllipse : Boolean; APenWidth : Integer; AFigureFilling : TFigureFilling;
      APenTexture : TTexture; ABrushTexture : TTexture);

  procedure IfLessThenExcange(var A1, A2 : Integer);
  var AX : Integer;
  begin
    if A2 < A1 then
      begin
        AX := A1;
        A1 := A2;
        A2 := AX;
      end;
  end;
  procedure ExcangeTexture();
  var TmpTexture : TTexture;
  begin
    TmpTexture := APenTexture;
    APenTexture := ABrushTexture;
    ABrushTexture := TmpTexture;
  end;
begin
  try
    if (AX1 = AX2) or (AY1 = AY2) then
      begin
        ADest.BeginUpdate();
        try
          ADest.Clear(0);
        finally
          ADest.EndUpdate();
          ADest.Changed();
          EMMS();
        end;
        Exit;
      end;

    IfLessThenExcange(AX1, AX2);
    IfLessThenExcange(AY1, AY2);
    if ((AX2 - AX1) <= (APenWidth * 2)) or
       ((AY2 - AY1) <= (APenWidth * 2)) then
      begin
        if (AFigureFilling = ffBorder) or (AFigureFilling = ffBorderAndFill) then
          begin
            AFigureFilling := ffFill;
            ExcangeTexture();
          end;
      end;

    // Fill  Border
    if AFigureFilling <> ffBorderAndFill then
      begin
        AAlphaBitmap.BeginUpdate();
        try
          AAlphaBitmap.Clear();
          if AFigureFilling = ffFill then
            begin
              if ADrawEllipse then
                gEllipse(AAlphaBitmap, FixedRect(Rect(AX1, AY1, AX2, AY2)), clWhite32,  pdoFastFilling)
              else
                AAlphaBitmap.FillRectS(AX1, AY1, AX2, AY2, clWhite32);
            end
          else
            begin
              if ADrawEllipse then
                begin
                  gEllipse(AAlphaBitmap, FixedRect(Rect(AX1, AY1, AX2, AY2)), clWhite32,  pdoFastFilling);
                  gEllipse(AAlphaBitmap, FixedRect(Rect(AX1 + APenWidth, AY1 + APenWidth, AX2 - APenWidth, AY2 - APenWidth)),
                           clBlack32,  pdoFastFilling);
                end
              else
                begin
                  AAlphaBitmap.FillRectS(AX1, AY1, AX2, AY2, clWhite32);
                  AAlphaBitmap.FillRectS(AX1 + APenWidth, AY1 + APenWidth, AX2 - APenWidth, AY2 - APenWidth, clBlack32);
                end;
            end;
        finally
          AAlphaBitmap.EndUpdate();
          AAlphaBitmap.Changed();
          EMMS();
        end;
        ADest.BeginUpdate();
        try
          if AFigureFilling = ffFill then
            TileTexure(ADest, ABrushTexture)
          else
            TileTexure(ADest, APenTexture);
          IntensityToAlpha(ADest, AAlphaBitmap);
        finally
          ADest.EndUpdate();
          ADest.Changed();
          EMMS();
        end;
      end
    else
      begin
       // ALT('1');
         // 
          AAlphaBitmap.BeginUpdate();
          try
            AAlphaBitmap.Clear();
            if ADrawEllipse then
              begin
                gEllipse(AAlphaBitmap, FixedRect(Rect(AX1 + APenWidth, AY1 + APenWidth, AX2 - APenWidth, AY2 - APenWidth)),
                         clWhite32,  pdoFastFilling);
              end
            else
              begin
                AAlphaBitmap.FillRectS(AX1 + APenWidth, AY1 + APenWidth, AX2 - APenWidth, AY2 - APenWidth, clWhite32);
              end;
          finally
            AAlphaBitmap.EndUpdate();
            AAlphaBitmap.Changed();
            EMMS();
          end;
         FDrawImageLayerBitmap.BeginUpdate();
         try
           TileTexure(FDrawImageLayerBitmap, ABrushTexture);
           IntensityToAlpha(FDrawImageLayerBitmap, AAlphaBitmap);
         finally
           FDrawImageLayerBitmap.EndUpdate();
           FDrawImageLayerBitmap.Changed();
         end;

         FDrawImage.Bitmap.BeginUpdate();
         try
           TileTexure(FDrawImage.Bitmap, APenTexture);
         finally
           FDrawImage.Bitmap.EndUpdate();
           FDrawImage.Bitmap.Changed();
           FDrawImage.Changed();
         end;
        ADest.BeginUpdate();
        try
          FDrawImage.PaintTo(ADest, Rect(0, 0, ADest.Width, ADest.Height));
          // 
          AAlphaBitmap.BeginUpdate();
          try
            AAlphaBitmap.Clear();
            if ADrawEllipse then
              begin
                gEllipse(AAlphaBitmap, FixedRect(Rect(AX1, AY1, AX2, AY2)), clWhite32,  pdoFastFilling);
              end
            else
              begin
                AAlphaBitmap.FillRectS(AX1, AY1, AX2, AY2, clWhite32);
              end;
          finally
            AAlphaBitmap.EndUpdate();
            AAlphaBitmap.Changed();
            EMMS();
          end;
          IntensityToAlpha(ADest, AAlphaBitmap);
        finally
          ADest.EndUpdate();
          ADest.Changed();
          EMMS();
        end;
      end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.DrawFigure()');
  end;
end;


procedure TdmCommonGraphics.DrawGraphicToBitmap(ADest: TBitmap32; ASource: TGraphic; ADestRect: TRect);
var TmpBmp : TBitmap32;
begin
  TmpBmp := TBitmap32.Create();
  try
    TmpBmp.Assign(ASource);
    TmpBmp.Resampler := TLinearResampler.Create();
    ADest.Draw(ADestRect, Rect(0, 0, TmpBmp.Width, TmpBmp.Height), TmpBmp);
  finally
    TmpBmp.Free();
  end;
end;

procedure TdmCommonGraphics.DrawLine(AX1, AY1, AX2, AY2: Integer; ADest, ABrush,
  AAlphaBitmap: TBitmap32; ATexture: TTexture);
var PointArray: TArrayOfPoint;
begin
  SetLength(PointArray, 2);
  PointArray[0].X := AX1;
  PointArray[0].Y := AY1;
  PointArray[1].X := AX2;
  PointArray[1].Y := AY2;
  DrawPolyLine(PointArray, ADest, ABrush, AAlphaBitmap, 0, ATexture);
end;

procedure TdmCommonGraphics.DrawPolyLine(APointArray: TArrayOfPoint; ADest, ABrush, AAlphaBitmap : TBitmap32;
      ASegmentsToDrawCount : Integer; ATexture : TTexture);
var OffsetX, OffsetY, PointArrayLength, i, j, k, CycleBegin : Integer;
begin
  try
    PointArrayLength := Length(APointArray);
    if PointArrayLength = 0 then
      Exit;
    if PointArrayLength = 1 then
      begin
        SetLength(APointArray, 2);
        PointArrayLength := 2;
        APointArray[1].X := APointArray[0].X;
        APointArray[1].Y := APointArray[0].Y;
      end;
    AAlphaBitmap.BeginUpdate();
    try
      if ASegmentsToDrawCount = 0 then
        begin
          AAlphaBitmap.Clear();
          CycleBegin := 0;
        end
      else
        begin
          CycleBegin := Max(0, (PointArrayLength - 1) - ASegmentsToDrawCount);
        end;

      OffsetX :=  Trunc(ABrush.Width / 2);
      OffsetY :=  Trunc(ABrush.Height / 2);
      for k := CycleBegin to PointArrayLength - 2 do
        for i := 0 to ABrush.Width - 1 do
          for j := 0 to ABrush.Height - 1 do
            begin
              AAlphaBitmap.PenColor := ABrush.Pixel[i, j];
              if not (TColor32Entry(AAlphaBitmap.PenColor).A = 0) then
                AAlphaBitmap.LineAS(
                  APointArray[k].X  + i - OffsetX, APointArray[k].Y + j - OffsetY,
                  APointArray[k + 1].X  + i - OffsetX, APointArray[k + 1].Y + j - OffsetY,
                  ABrush.Pixel[i, j], True);
            end;
    finally
      AAlphaBitmap.EndUpdate();
      AAlphaBitmap.Changed();
      EMMS();
    end;
    ADest.BeginUpdate();
    try
      TileTexure(ADest, ATexture);
      IntensityToAlpha(ADest, AAlphaBitmap);
    finally
      ADest.EndUpdate();
      ADest.Changed();
      EMMS();
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.DrawPolyLine()');
  end;
end;

procedure TdmCommonGraphics.DrawStamp(AX, AY, AStampSize : Integer; ADest, AStamp,
  AAlphaBitmap: TBitmap32; ATexture: TTexture);
begin
  try
    AAlphaBitmap.BeginUpdate();
    try
      AAlphaBitmap.Clear(clBlack32);
      AX := AX - (AStampSize div 2);
      AY := AY - (AStampSize div 2);
      AAlphaBitmap.Draw(Rect(AX, AY, AX + AStampSize, AY + AStampSize), Rect(0, 0, AStamp.Width, AStamp.Height), AStamp);
    finally
      AAlphaBitmap.EndUpdate();
      AAlphaBitmap.Changed();
      EMMS();
    end;
    ADest.BeginUpdate();
    try
      TileTexure(ADest, ATexture);
      IntensityToAlpha(ADest, AAlphaBitmap);
    finally
      ADest.EndUpdate();
      ADest.Changed();
      EMMS();
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.DrawStamp()');
  end;
end;

procedure TdmCommonGraphics.DrawText(AX, AY: Integer; ADest,
  AAlphaBitmap: TBitmap32; AText : string;
  ATexture: TTexture; ADrawCursor: Boolean);
var TxtSize : TSize;
begin
  try
    if Length(AText) = 0 then
      begin
        ADest.Clear(0);
        if not ADrawCursor then
          Exit;
        TxtSize.cx := 0;
        TxtSize.cy := ADest.TextHeight('_|');
      end
    else
      begin
        AAlphaBitmap.BeginUpdate();
        try
          AAlphaBitmap.Clear(clBlack32);
          AAlphaBitmap.Font := ADest.Font;
          TxtSize := AAlphaBitmap.RenderText(AX, AY, AText, 1, clWhite32);
        finally
          AAlphaBitmap.EndUpdate();
          AAlphaBitmap.Changed();
          EMMS();
        end;
      end;
    ADest.BeginUpdate();
    try
      if TxtSize.cx > 0 then
        begin
          TileTexure(ADest, ATexture);
          IntensityToAlpha(ADest, AAlphaBitmap);
        end;
      if ADrawCursor then
        begin
          ADest.VertLineS(AX + TxtSize.cx , AY, AY + TxtSize.cy, clBlack32);
          ADest.VertLineS(AX + TxtSize.cx + 1, AY, AY + TxtSize.cy, clWhite32);
        end;
    finally
      ADest.EndUpdate();
      ADest.Changed();
      EMMS();
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.DrawText()');
  end;
end;
//linear algoritm in development
procedure TdmCommonGraphics.FloodFill1(AX, AY, ATolerance: Integer;
  ASourceImage: TImage32; ADest: TBitmap32; ATexture: TTexture);
var ERMin, ERMax, EBMin, EBMax, EGMin, EGMax : Byte;
    AlphaBitmap, BitmapWithImage : TBitmap32;
    PointsToProcess : array of TPoint;
    NeedAddTop, NeedAddBotom : Boolean;

  procedure AddPoint(AX, AY : Integer);
  var LengthPTP : Integer;
  begin
    LengthPTP := Length(PointsToProcess);
    SetLength(PointsToProcess, LengthPTP + 1);
    PointsToProcess[LengthPTP].X := AX;
    PointsToProcess[LengthPTP].Y := AY;
//    AlphaBitmap.Pixel[AX, AY] := clWhite32;
  end;

  function Similar(ASourceColor : TColor32) : Boolean;
  var SR, SG, SB : Byte;
  begin
//       Result := (TColor32Entry(ASourceColor).R in [ERMin..ERMax]) and
//                 (TColor32Entry(ASourceColor).G in [EGMin..EGMax]) and
//                 (TColor32Entry(ASourceColor).B in [EBMin..EBMax]);

    Color32ToRGB(ASourceColor,   SR, SG, SB);
    Result := (SR in [ERMin..ERMax]) and
              (SG in [EGMin..EGMax]) and
              (SB in [EBMin..EBMax]);
  end;

  function ProcessPoint(AX2, AY2 : Integer) : Boolean;
  begin
    Result := (AX2 >= 0) and (AY2 >= 0) and (AX2 < AlphaBitmap.Width) and (AY2 < AlphaBitmap.Height)
     and (AlphaBitmap.Pixel[AX2, AY2] = clBlack32) and Similar( BitmapWithImage.Pixel[AX2, AY2]);
  end;

  procedure SetNextPoint(X1, Y1: Integer);
  begin
    NeedAddTop := True;
    NeedAddBotom := True;
    //  
    while ProcessPoint(X1, Y1) do
      begin
        AlphaBitmap.Pixel[X1, Y1] := clWhite32;

        if NeedAddTop then
        begin
          if ProcessPoint(X1, Y1 - 1) then
            begin
              NeedAddTop := False;
              AddPoint(X1, Y1 - 1);
            end;
        end
        else
          NeedAddTop := not ProcessPoint(X1, Y1 - 1);

        if NeedAddBotom then
        begin
          if ProcessPoint(X1, Y1 + 1) then
            begin
              NeedAddBotom := False;
              AddPoint(X1, Y1 + 1);
            end;
        end
        else
          NeedAddBotom := not ProcessPoint(X1, Y1 + 1);

        Inc(X1);
      end;
  end;

begin
  Assert('False');
  {try
    BitmapWithImage := TBitmap32.Create();
    AlphaBitmap := TBitmap32.Create();
    try
      BitmapWithImage.SetSizeFrom(ASourceImage);
      AlphaBitmap.SetSizeFrom(BitmapWithImage);

      ASourceImage.PaintTo(BitmapWithImage, Rect(0, 0, BitmapWithImage.Width, BitmapWithImage.Height));
      AlphaBitmap.FillRectS(0,0, AlphaBitmap.Width, AlphaBitmap.Height, clBlack32);

      //     
      EtalonColor := BitmapWithImage.Pixels[AX, AY];
      Color32ToRGB(EtalonColor, ERMin, EGMin, EBMin);
      ERMax := Min(ERMin + ATolerance, $FF);
      ERMin := Max(ERMin - ATolerance, $00);
      EGMax := Min(EGMin + ATolerance, $FF);
      EGMin := Max(EGMin - ATolerance, $00);
      EBMax := Min(EBMin + ATolerance, $FF);
      EBMin := Max(EBMin - ATolerance, $00);

      SetLength(PointsToProcess, 0);
      PointsToProcessCounter := 0;
      AlphaBitmap.BeginUpdate();
      ALT('1');
      try
        //   
        while (AX > 0) and Similar(BitmapWithImage.Pixel[AX, AY]) do
          Dec(AX);
        //    
        SetNextPoint(AX, AY);
        while (PointsToProcessCounter < Length(PointsToProcess)) do
          begin
            SetNextPoint(PointsToProcess[PointsToProcessCounter].X , PointsToProcess[PointsToProcessCounter].Y);
            Inc(PointsToProcessCounter);
          end;
      finally
        ALT('2');
        AlphaBitmap.EndUpdate();
        AddLogStr('_' + IntToStr(Length(PointsToProcess)));
        SetLength(PointsToProcess, 0);
        AlphaBitmap.Changed();
        EMMS();
      end;
      ADest.BeginUpdate();
      try
        TileTexure(ADest, ATexture);
        IntensityToAlpha(ADest, AlphaBitmap);
      finally
        ADest.EndUpdate();
        ADest.Changed();
        EMMS();
      end;
    finally
      BitmapWithImage.Free();
      AlphaBitmap.Free();
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.FloodFill()');
  end;}
end;

function TdmCommonGraphics.GetBitmapFromListOrRes(AList: TList; AIndex: Integer; AResPrefix: String): TBitmap32;
var ResStream : TResourceStream;
begin
  Result := nil;
  if (AIndex < 0) or (AIndex >= AList.Count) then
  begin
    AddLogStr('AIndex out of range ' + AResPrefix + ' ' + IntToStr(AIndex), 'TdmCommonGraphics.GetBitmapFromListOrRes()');
    Exit;
  end;
  if Assigned(AList[AIndex]) then
  begin
    Result := TBitmap32(AList[AIndex]);

  end
  else
  begin
    Result := TBitmap32.Create();
    ResStream := TResourceStream.Create(HInstance, AResPrefix + Format('%.3d', [AIndex]), RT_RCDATA);
    try
      Result.LoadFromStream(ResStream);
    finally
      FreeAndNil(ResStream);
    end;
    AList[AIndex] := Result;
  end;
end;

function TdmCommonGraphics.GetBrush(AIndex: Integer): TBitmap32;
begin
  Result := GetBitmapFromListOrRes(FBrushesList, AIndex, 'B');
end;

function TdmCommonGraphics.GetStamp(AIndex: Integer): TBitmap32;
begin
  Result := GetBitmapFromListOrRes(FStampList, AIndex, 'S');
end;

function TdmCommonGraphics.GetTexture(AIndex: Integer): TBitmap32;
begin
  Result := GetBitmapFromListOrRes(FTextureList, AIndex, 'T');
end;

procedure TdmCommonGraphics.FloodFill(AX, AY, ATolerance: Integer;
  ASourceImage: TImage32; ADest: TBitmap32; ATexture: TTexture);
var EtalonColor : TColor32;
    ERMin, ERMax, EBMin, EBMax, EGMin, EGMax : Byte;
    AlphaBitmap, BitmapWithImage : TBitmap32;
    PointsToProcess : array of TPoint;
    PointsToProcessCounter : Integer;

  procedure AddPoint(AX, AY : Integer);
  var LengthPTP : Integer;
  begin
    LengthPTP := Length(PointsToProcess);
    SetLength(PointsToProcess, LengthPTP + 1);
    PointsToProcess[LengthPTP].X := AX;
    PointsToProcess[LengthPTP].Y := AY;
    AlphaBitmap.Pixel[AX, AY] := clWhite32;
  end;

  procedure SetNextPoint(X1, Y1: Integer);
    function Similar(ASourceColor : TColor32) : Boolean;
    var SR, SG, SB : Byte;
    begin
//       Result := (TColor32Entry(ASourceColor).R in [ERMin..ERMax]) and
//                 (TColor32Entry(ASourceColor).G in [EGMin..EGMax]) and
//                 (TColor32Entry(ASourceColor).B in [EBMin..EBMax]);

      Color32ToRGB(ASourceColor,   SR, SG, SB);
      Result := (SR in [ERMin..ERMax]) and
                (SG in [EGMin..EGMax]) and
                (SB in [EBMin..EBMax]);
    end;
    procedure ProcessPoint(AX2, AY2 : Integer);
    begin
      if (AX2 >= 0) and (AY2 >= 0) and (AX2 < AlphaBitmap.Width) and (AY2 < AlphaBitmap.Height)
       and (AlphaBitmap.Pixel[AX2, AY2] = clBlack32) and Similar( BitmapWithImage.Pixel[AX2, AY2]) then
        AddPoint(AX2, AY2);
    end;
  begin
    ProcessPoint(X1,   Y1-1);
    ProcessPoint(X1+1, Y1-1);
    ProcessPoint(X1+1, Y1);
    ProcessPoint(X1+1, Y1+1);
    ProcessPoint(X1,   Y1+1);
    ProcessPoint(X1-1, Y1+1);
    ProcessPoint(X1-1, Y1);
    ProcessPoint(X1-1, Y1-1);
  end;

begin
  try
    BitmapWithImage := TBitmap32.Create();
    AlphaBitmap := TBitmap32.Create();
    try
      BitmapWithImage.SetSizeFrom(ASourceImage);
      AlphaBitmap.SetSizeFrom(BitmapWithImage);

      ASourceImage.PaintTo(BitmapWithImage, Rect(0, 0, BitmapWithImage.Width, BitmapWithImage.Height));
      AlphaBitmap.FillRectS(0,0, AlphaBitmap.Width, AlphaBitmap.Height, clBlack32);

      //     
      EtalonColor := BitmapWithImage.Pixels[AX, AY];
      Color32ToRGB(EtalonColor, ERMin, EGMin, EBMin);
      ERMax := Min(ERMin + ATolerance, $FF);
      ERMin := Max(ERMin - ATolerance, $00);
      EGMax := Min(EGMin + ATolerance, $FF);
      EGMin := Max(EGMin - ATolerance, $00);
      EBMax := Min(EBMin + ATolerance, $FF);
      EBMin := Max(EBMin - ATolerance, $00);

      SetLength(PointsToProcess, 0);
      PointsToProcessCounter := 0;
      AlphaBitmap.BeginUpdate();
//      ALT('1');
      try
        AddPoint(AX, AY);
        while (PointsToProcessCounter < Length(PointsToProcess)) do
          begin
            SetNextPoint(PointsToProcess[PointsToProcessCounter].X , PointsToProcess[PointsToProcessCounter].Y);
            Inc(PointsToProcessCounter);
          end;
      finally
//        ALT('2');
        AlphaBitmap.EndUpdate();
//        AddLogStr('_' + IntToStr(Length(PointsToProcess)));
        SetLength(PointsToProcess, 0);
        AlphaBitmap.Changed();
        EMMS();
      end;
      ADest.BeginUpdate();
      try
        TileTexure(ADest, ATexture);
        IntensityToAlpha(ADest, AlphaBitmap);
      finally
        ADest.EndUpdate();
        ADest.Changed();
        EMMS();
      end;
    finally
      BitmapWithImage.Free();
      AlphaBitmap.Free();
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.FloodFill()');
  end;
end;

procedure TdmCommonGraphics.Resize(ADest: TBitmap32; ANewSize: Integer);
var TmpBmp : TBitmap32;
begin
  TmpBmp := TBitmap32.Create();
  try
    TmpBmp.Assign(ADest);
    TmpBmp.Resampler := TLinearResampler.Create();
    ADest.SetSize(ANewSize, ANewSize);
    ADest.Draw(Rect(0, 0, ADest.Width, ADest.Height), Rect(0, 0, TmpBmp.Width, TmpBmp.Height), TmpBmp);
  finally
    TmpBmp.Free();
  end;
end;


//procedure TdmCommonGraphics.PC_Modulate(F: TColor32; var B: TColor32;
//  M: TColor32);
//var
//  Xe: TColor32Entry absolute F;
//  Ye: TColor32Entry absolute B;
//  Af, Ab: PByteArray;
//begin
// // b := b + AlphaComponent(F);
//  b := f;
//  TColor32Entry(b).A := 255;
//  TColor32Entry(b).G := 0;
//  TColor32Entry(b).B := 0;
//  TColor32Entry(b).R := 0;
//Exit;
//
//  if AlphaComponent(F) = 0 then
//    exit;
//  if AlphaComponent(F) = 255 then
//    begin
//      B := F;
//      exit;
//    end;
//
//  b := ColorAdd(F, B);
//
//  exit;
//  if M = 0 then
//  begin
//    Exit;
//  end;
//
//  if M >= $FF then
//  begin
//    B := F;
//    Exit;
//  end;
//
//  with Xe do
//  begin
//    Af := @DivTable[M];
//    Ab := @DivTable[M];
//    R := Ab[Ye.R] + Af[R];
//    G := Ab[Ye.G] + Af[G];
//    B := Ab[Ye.B] + Af[B];
//  end;
//  B := F;
//end;

procedure TdmCommonGraphics.TileTexure(ADest : TBitmap32; ATexture: TTexture);
var TileXCount, TileYCount, i, j : Integer;
begin
  try
    if not ATexture.IsColor then
      begin
        TileXCount  := Ceil(ADest.Width / ATexture.TextureBitmap.Width);
        TileYCount  := Ceil(ADest.Height / ATexture.TextureBitmap.Height);
        for i := 0 to TileXCount - 1 do
          for j := 0 to TileYCount - 1 do
             ADest.Draw(i * ATexture.TextureBitmap.Width, j * ATexture.TextureBitmap.Height, ATexture.TextureBitmap);
      end
    else
      begin
        ADest.FillRect(0, 0, ADest.Width, ADest.Height, ATexture.TextureColor);
      end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TdmCommonGraphics.TileTexure()');
  end;
end;

{ TTexture }

constructor TTexture.Create;
begin
  inherited;
  FTextureColor := clBlack;
  FTextureBitmapIndex := -1;
  FIsColor := True;
  FTextureBitmap := TBitmap32.Create();
end;

procedure TTexture.DecodeTexture(AInputString: String);
begin
  if Length(AInputString) < 6 then
    Exit;
  SetLength(AInputString, 6);
  if AInputString[1] = 'T' then
    begin
      Self.TextureBitmapIndex := StrToIntDef('$' + Copy(AInputString, 2,2), -1);
    end
  else
    begin
      Self.TextureColor := Color32(StrToIntDef('$' + AInputString, clWhite));
    end;
end;

destructor TTexture.Destroy;
begin
  FreeAndNil(FTextureBitmap);
  inherited;
end;

function TTexture.EncodeTexture: String;
begin
  if Self.TextureBitmapIndex < 0 then
    begin
      Result := IntToHex(WinColor(Self.TextureColor),6);
    end
  else
    begin
      Result := 'T' + IntToHex(Self.TextureBitmapIndex, 2) + IntToHex(WinColor(Self.TextureColor), 6);
      SetLength(Result, 6);
    end;
end;

procedure TTexture.SetIsColor(const Value: Boolean);
begin
  FIsColor := Value;
end;

procedure TTexture.SetTextureBitmap(const Value: TBitmap32);
begin
  FTextureBitmap.BeginUpdate();
  try
    FTextureBitmap.SetSizeFrom(Value);
    FTextureBitmap.Draw(0, 0, Value);
  finally
    FTextureBitmap.EndUpdate();
    FTextureBitmap.Changed();
    IsColor := False;
  end;
end;


procedure TTexture.SetTextureBitmapIndex(const Value: Integer);
var TmpBitmap : TBitmap32;
begin
  TmpBitmap := dmCommonGraphics.GetTexture(Value);
  if Assigned(TmpBitmap) then
  begin
    Self.TextureBitmap := TmpBitmap;
    FTextureBitmapIndex := Value;
  end
  else
    Self.TextureColor := Color32(clRed);
end;

procedure TTexture.SetTextureColor(const Value: TColor32);
begin
  FTextureColor := Value;
  FTextureBitmapIndex := -1;
  IsColor := True;
end;

{ TMyBitmap32 }

procedure TMyBitmap32.SetTag(const Value: Integer);
begin
  FTag := Value;
end;

end.
