unit uFunc;

interface

uses Windows, Classes, SysUtils, Messages, Graphics, uConst, IniFiles, StrUtils;

type
  TCommFortProcess = procedure(dwPluginID : DWORD; dwID: DWORD; bOutBuffer : PAnsiChar; dwOutBufferSize : DWORD); stdcall;
  TCommFortGetData = function(dwPluginID : DWORD; dwID : DWORD; bInBuffer : PAnsiChar; dwInBufferSize : DWORD; bOutBuffer : PAnsiChar; dwOutBufferSize : DWORD): DWORD; stdcall;

function Str32xToInt(Str : String) : Integer;
function IntToStr32x(Number : Integer) : String;
procedure ClearImage(Image : TBitMap);
function LeftCutStr(var AInStr : string; ACutCount : Integer): String;

function  fReadInteger(bInBuffer : PAnsiChar; var iOffset : Integer): Integer;
function  fReadDate(bInBuffer : PAnsiChar; var iOffset : Integer): TDateTime;
function  fReadText(bInBuffer : PAnsiChar; var iOffset : Integer): WideString;
procedure pReadStream(bInBuffer : PAnsiChar; var iOffset : Integer; sOutStream : TStream);

procedure fWriteInteger(var bOutBuffer : PAnsiChar; var iOffset  : Integer; iValue : Integer);
procedure fWriteText(bOutBuffer : PAnsiChar; var iOffset  : Integer; uValue : WideString);
function  fTextToAnsiString(uText : WideString) : AnsiString;
function  fIntegerToAnsiString(iValue : Integer) : AnsiString;
function fStreamToAnsiString(sStream : TStream) : AnsiString; //      

function GetFullSettingsFileName(): string;
function GetCurrentUserName(): string;

procedure MsgToChannel(AChannel, AText : String; const ANormalMessage : Boolean = True);
procedure MsgToPrivat(AUserName, AText : String; const ANormalMessage : Boolean = True);
procedure CreateOrEnterChannel(const AVUserName, AChannel: String; const AVisible : Boolean = True; const APrivate : Boolean = False);
procedure LeaveChannel(AUserOrChanName: String; APrivate : Boolean = False);
function GetCurrentChannelName(): string;
procedure AddLogStr(AText : String; AFunctionName : String = ''; const AError : Boolean = True);
{$IFDEF DEBUG}
procedure ALT(AFunctionName : String = '');
{$ENDIF}
{$IFNDEF VTALKING}
procedure SendPaintData(AUserOrChanName, APaintData : String);
{$ENDIF}

procedure ImageToChannel(AUserOrChanName : String; AStream : TStream);
function GetImageLimitation(AUserOrChanName : String): Integer;


procedure FreeAndNilA(var Obj);
function IsValidPaintDataString(AData : String): Boolean; inline;
function PlugVersion() : string;

function IsPluginExpired() : Boolean;
function GetDefaultBrowserPath(): String;
procedure OpenUrlInDefaultBrowser(AUrl : String);

var
  dwPluginID : DWORD;
  CommFortProcess : TCommFortProcess;
  CommFortGetData : TCommFortGetData;
  MemIniFile : TMemIniFile = nil;
  DefaultBrowserPath : string = '';
  LastTickCount : Dword;
  LogFile : Text;
  LogFileName : String;
  {$IFDEF VTALKING}
  SendPaintData : TSendPaintData;
  {$ENDIF}

implementation

uses DateUtils, ShellAPI, SHFolder;

//--------------------------------------------------------------------------------------------------

function IntToStr32x(Number : Integer) : String;
var i : integer;
begin
  i := Number div 32;
  Result := Char(i+1040) + Char(Number - i * 32 + 1040);
end;
//--------------------------------------------------------------------------------------------------
function Str32xToInt(Str : String) : Integer;
begin
  if Length(Str) >= 2 then
    Result := (Integer(Str[1]) - 1040) * 32 + Integer(Str[2]) - 1040
  else
    Result := 0;
end;
//--------------------------------------------------------------------------------------------------
function GetIniFileName : string;
var thefilename: array[0..max_path] of char;
begin
  fillchar(thefilename, sizeof(thefilename), #0);
  getmodulefilename(hinstance, thefilename, sizeof(thefilename));
  Result := ExtractFileName(TheFileName);
  Result := ChangeFileExt(Result, '.ini');
end;
//--------------------------------------------------------------------------------------------------
procedure ClearImage(Image : TBitMap);
begin
  with Image.Canvas do
    begin
      Pen.Color := clWhite;
      Brush.Color := clWhite;
      Rectangle(0, 0, Image.Width + 1, Image.Height + 1);
    end;
end;

function LeftCutStr(var AInStr : string; ACutCount : Integer): String;
begin
  Result := LeftStr(AInStr, ACutCount);
  Delete(AInStr, 1, ACutCount);
end;

//---------------------------------------------------------------------------
function fReadInteger(bInBuffer : PAnsiChar; var iOffset : Integer): Integer; //       
begin
	CopyMemory(@Result, bInBuffer + iOffSet, 4);
	iOffset := iOffset + 4;
end;

function  fReadDate(bInBuffer : PAnsiChar; var iOffset : Integer): TDateTime;
begin
	CopyMemory(@Result, bInBuffer + iOffSet, 8);
	iOffset := iOffset + 8;
end;

function fReadText(bInBuffer : PAnsiChar; var iOffset : Integer): WideString; //       
 var iLength : Integer;
begin
	CopyMemory(@iLength, bInBuffer + iOffSet, 4);
	iOffset := iOffset + 4;
	SetLength(Result, iLength);
	CopyMemory(@Result[1], bInBuffer + iOffSet, iLength * 2);
	iOffset := iOffset + iLength * 2;
end;

procedure pReadStream(bInBuffer : PAnsiChar; var iOffset : Integer; sOutStream : TStream);
 var iLength : Integer;
begin
	CopyMemory(@iLength, bInBuffer + iOffSet, 4);
	iOffset := iOffset + 4;
  sOutStream.Write((bInBuffer + iOffSet)^, iLength);
end;
//---------------------------------------------------------------------------
procedure fWriteInteger(var bOutBuffer : PAnsiChar; var iOffset  : Integer; iValue : Integer); //       
begin
	CopyMemory(bOutBuffer + iOffSet, @iValue, 4);
	iOffset := iOffset + 4;
end;
//---------------------------------------------------------------------------
procedure fWriteText(bOutBuffer : PAnsiChar; var iOffset  : Integer; uValue : WideString); //       
	var iLength : Integer;
begin
	iLength := Length(uValue);
	CopyMemory(bOutBuffer + iOffset, @iLength, 4);
	iOffset := iOffset + 4;

	CopyMemory(bOutBuffer + iOffSet, @uValue[1], iLength * 2);
	iOffset := iOffset + iLength * 2;
end;

//---------------------------------------------------------------------------
function fTextToAnsiString(uText : WideString) : AnsiString; //      
	var iLength : Integer;
begin
	iLength := Length(uText);

	SetLength(Result, 4 + iLength * 2);

	CopyMemory(@Result[1], @iLength, 4);
	CopyMemory(PAnsiChar(Result) + 4, @uText[1], iLength * 2);
end;

function fStreamToAnsiString(sStream : TStream) : AnsiString; //      
	var iLength : Integer;
begin
	iLength := sStream.Size;
  sStream.Position := 0;
	SetLength(Result, 4 + iLength * 2);

	CopyMemory(@Result[1], @iLength, 4);
  sStream.Read((PAnsiChar(@Result[1]) + 4)^, iLength);
end;

//---------------------------------------------------------------------------
function fIntegerToAnsiString(iValue : Integer) : AnsiString; //      
begin
	SetLength(Result, 4);
	CopyMemory(@Result[1], @iValue, 4);
end;

{$IFDEF VTALKING}
function GetFullSettingsFileName(): string;
const SHGFP_TYPE_CURRENT = 0;
var  path: array [0..MAX_PATH] of char;
begin
  SHGetFolderPath(0,CSIDL_APPDATA,0,SHGFP_TYPE_CURRENT,@path[0]);
  Result := IncludeTrailingPathDelimiter(path) + INIFILE;
  if not FileExists(Result) then
    ForceDirectories(ExtractFileDir(Result));
  LogFileName := ChangeFileExt(Result, '.log');
  AssignFile(LogFile, LogFileName);
  Rewrite(LogFile);
  CloseFile(LogFile);
end;
{$ELSE}
function GetFullSettingsFileName(): string;
var iSize : Integer;
    aData : AnsiString;
    PaData : PAnsiChar;
    iReadOffset : Integer;
begin
  SetLength(Result, 0);
	iSize := CommFortGetData(dwPluginID, 2010, nil, 0, nil, 0);
  if iSize > 0 then
    begin
      SetLength(aData, iSize);
      PaData := PAnsiChar(aData);
      iSize := CommFortGetData(dwPluginID, 2010, PaData, iSize, nil, 0);
      SetLength(aData, iSize);
      iReadOffset := 0;
      Result := fReadText(PaData, iReadOffset);
    end;
  Result := IncludeTrailingPathDelimiter(Result) + INIFILE;
end;
{$ENDIF}
function GetCurrentUserName(): string;
var iSize : Integer;
    aData : AnsiString;
    PaData : PAnsiChar;
    iReadOffset : Integer;
begin
  SetLength(Result, 0);
	iSize := CommFortGetData(dwPluginID, 12, nil, 0, nil, 0);
  if iSize > 0 then
    begin
      SetLength(aData, iSize);
      PaData := PAnsiChar(aData);
      iSize := CommFortGetData(dwPluginID, 12, PaData, iSize, nil, 0);
      SetLength(aData, iSize);
      iReadOffset := 0;
      Result := fReadText(PaData, iReadOffset);
    end;
end;

procedure CreateOrEnterChannel(const AVUserName, AChannel: String; const AVisible : Boolean = True; const APrivate : Boolean = False);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AVUserName)+
   fTextToAnsiString(AChannel)+
   fIntegerToAnsiString(Integer(AVisible))+
   fIntegerToAnsiString(Integer(APrivate));
	CommFortProcess(dwPluginID, 1026, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure LeaveChannel(AUserOrChanName: String; APrivate : Boolean = False);
var aDataToSend : AnsiString;
    ExitCode : Integer;
begin
  if Length(AUserOrChanName) = 0 then
    Exit;
  if AUserOrChanName[1] = '&' then
    Delete(AUserOrChanName, 1, 1);
	aDataToSend := fTextToAnsiString(AUserOrChanName);
  if APrivate then ExitCode := 65 else ExitCode := 66;
	CommFortProcess(dwPluginID, ExitCode, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure MsgToChannel(AChannel, AText : String; const ANormalMessage : Boolean = True);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AChannel) +
   fIntegerToAnsiString(Integer(not ANormalMessage)) +
 	 fTextToAnsiString(AText);
	CommFortProcess(dwPluginID, 50, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

procedure MsgToPrivat(AUserName, AText : String; const ANormalMessage : Boolean = True);
var aDataToSend : AnsiString;
begin
	aDataToSend := fTextToAnsiString(AUserName) +
   fIntegerToAnsiString(Integer(not ANormalMessage)) +
 	 fTextToAnsiString(AText);
	CommFortProcess(dwPluginID, 63, PAnsiChar(aDataToSend), Length(aDataToSend));
end;

function GetCurrentChannelName(): string;
var iSize : Integer;
    aData : AnsiString;
    PaData : PAnsiChar;
    iReadOffset : Integer;
begin
  SetLength(Result, 0);
	iSize := CommFortGetData(dwPluginID, 14, nil, 0, nil, 0);
  if iSize > 0 then
    begin
      SetLength(aData, iSize);
      PaData := PAnsiChar(aData);
      iSize := CommFortGetData(dwPluginID, 14, PaData, iSize, nil, 0);
      SetLength(aData, iSize);
      iReadOffset := 0;
      Result := fReadText(PaData, iReadOffset);
    end;
end;

procedure AddLogStr(AText : String; AFunctionName : String = ''; const AError : Boolean = True);
var aDataToSend : AnsiString;
begin
  OutputDebugString(PChar(AFunctionName + ': ' + AText));
  {$IFDEF VTALKING}
  {$IFNDEF DEBUG}
  if AError then
  {$ENDIF}
  begin
    try
      AssignFile(LogFile, LogFileName);
      Append(LogFile);
      Writeln(LogFile, '[' + DateTimeToStr(Now()) + '] ' + AFunctionName + ': ' + AText);
      CloseFile(LogFile);
    except
    end;
  end;
  {$ELSE}
	aDataToSend := fIntegerToAnsiString(Integer(AError) * 2) +
	 fIntegerToAnsiString(0) +
	 fIntegerToAnsiString(0) +
 	 fTextToAnsiString('PaintBoard ' + AFunctionName + ': ' + AText);
	CommFortProcess(dwPluginID, 100, PAnsiChar(aDataToSend), Length(aDataToSend));
 {$ENDIF}
end;


{$IFDEF DEBUG}
procedure ALT(AFunctionName : String = '');
begin
  AddLogStr(IntToStr(GetTickCount() - LastTickCount), AFunctionName, False);
  LastTickCount := GetTickCount();
end;
{$ENDIF}

{$IFNDEF VTALKING}
procedure SendPaintData(AUserOrChanName, APaintData : String);
begin
  APaintData := START_PAINT_MESSAGE + APaintData + END_PAINT_MESSAGE;
  if AUserOrChanName[1] = '&' then
    begin
      Delete(AUserOrChanName, 1, 1);
      MsgToPrivat(AUserOrChanName, APaintData);
    end
  else
    MsgToChannel(AUserOrChanName, APaintData);
end;
{$ENDIF}

procedure ImageToChannel(AUserOrChanName : String; AStream : TStream);
var aDataToSend : AnsiString;
    iSendCode : Integer;
begin
  if AUserOrChanName[1] = '&' then
  begin
    Delete(AUserOrChanName, 1, 1);
    iSendCode := 64;
  end
  else
    iSendCode := 51;

  aDataToSend := fTextToAnsiString(AUserOrChanName) +
   fIntegerToAnsiString(0) + //0 - type bmp
   fStreamToAnsiString(AStream);
  try
    CommFortProcess(dwPluginID, iSendCode, PAnsiChar(aDataToSend), Length(aDataToSend));
  except
     on E : Exception do
       AddLogStr(E.Message, 'ImageToChannel()');
  end;
end;

function GetImageLimitation(AUserOrChanName : String): Integer;
const ModeToPixelsArray : array[0..5] of Integer = (0, 0, 30000, 60000, 125000, 250000);
var iSize : Integer;
    aData : AnsiString;
    PaData : PAnsiChar;

    outData : AnsiString;
    iReadOffset : Integer;
begin
  if AUserOrChanName[1] = '&' then  //    
    Exit(ModeToPixelsArray[5]);
  outData := fTextToAnsiString(AUserOrChanName);
	iSize := CommFortGetData(dwPluginID, 21, nil, 0, PAnsiChar(outData), Length(outData));
  if iSize > 0 then
  begin
    SetLength(aData, iSize);
    PaData := PAnsiChar(aData);

    iSize := CommFortGetData(dwPluginID, 21, PaData, iSize, PAnsiChar(outData), Length(outData));
    SetLength(aData, iSize);
    iReadOffset := 0;
    fReadText(PaData, iReadOffset);  //( )
    fReadText(PaData, iReadOffset);  //(  ,  )
    fReadDate(PaData, iReadOffset);  //__(  )
    fReadText(PaData, iReadOffset);  //()
    Result := fReadInteger(PaData, iReadOffset);  //(  )
    if (Result < 0) or (Result > 5) then
      Result := 0
    else
      Result := ModeToPixelsArray[Result];
  end
  else
    Result := 0
end;


procedure FreeAndNilA(var Obj);
begin
  if Assigned(Pointer(Obj)) then
    FreeAndNil(Obj);
end;

function IsValidPaintDataString(AData : String): Boolean;
begin
  Result := StartsText(START_PAINT_MESSAGE, AData) and
    EndsText(END_PAINT_MESSAGE, AData);
end;

function PlugVersion() : string;
const NOVIDATA = '';
var
  dwInfoSize,           // Size of VERSIONINFO structure
  dwVerSize,            // Size of Version Info Data
  dwWnd: DWORD;         // Handle for the size call.
  FI: PVSFixedFileInfo; // Delphi structure; see WINDOWS.PAS
  ptrVerBuf: Pointer;   // pointer to a version buffer
  strFileName,          // Name of the file to check
  strVersion : string;  // Holds parsed version number
  thefilename: array[0..max_path] of char;
begin
  fillchar(thefilename, sizeof(thefilename), #0);
  getmodulefilename(hinstance, thefilename, sizeof(thefilename));
  strFileName := thefilename;
  dwInfoSize := getFileVersionInfoSize( pChar( strFileName ), dwWnd);
  if ( dwInfoSize = 0 ) then
     result := NOVIDATA
  else
    begin
      getMem( ptrVerBuf, dwInfoSize );
      try
        if getFileVersionInfo( pChar( strFileName ), dwWnd, dwInfoSize, ptrVerBuf ) then
          if verQueryValue( ptrVerBuf, '\', pointer(FI), dwVerSize ) then
            strVersion := Format( '%d.%d.%d',
                        [hiWord( FI.dwFileVersionMS ),
                         loWord( FI.dwFileVersionMS ),
                         hiWord( FI.dwFileVersionLS )]);
      finally
        freeMem( ptrVerBuf );
      end;
    end;
  Result := strVersion;
end;

function IsPluginExpired() : Boolean;
//var TmpDate : TDateTime;
begin
  Result := False;
//  TmpDate := Today();
//  Result :=  (YearOf(TmpDate) >= EXPIRE_YEAR) and (MonthOf(TmpDate) >= EXPIRE_MONTH);
end;

function GetDefaultBrowserPath(): String;
var
 tmp: PChar;
 res: LPTSTR;
 TestFileHandle  : Integer;
 TestTmpFileName,
 HtmlFileName    : String;
begin
 Result := 'explorer';
 Randomize;
 HtmlFileName := IntToStr(Random(100000)) + '.htm';
 tmp := StrAlloc(255);
 res := StrAlloc(255);
 try
  GetTempPath(255, tmp);
  TestTmpFileName := tmp + HtmlFileName;
  SysUtils.DeleteFile(TestTmpFileName);
  TestFileHandle := FileCreate(TestTmpFileName);
  if ( TestFileHandle <> -1 ) then
   begin
    try
     if ( FindExecutable(PChar(HtmlFileName), tmp, res) > 32 ) then
      begin
        Result := String(Res);
        if not FileExists(' 1' +Result) then
          Result := 'explorer';
      end;
    finally
     CloseHandle(TestFileHandle);
     SysUtils.DeleteFile(TestTmpFileName);
    end;
   end;
 finally
  StrDispose(tmp);
  StrDispose(res);
 end;
end;

procedure OpenUrlInDefaultBrowser(AUrl : String);
begin
  if Length(DefaultBrowserPath) = 0 then
    DefaultBrowserPath := GetDefaultBrowserPath();
  if Length(AUrl) > 0 then
    begin
      ShellExecute(0, 'open', PChar(DefaultBrowserPath), PChar('"' + AUrl + '"'), '', sw_Show);
    end;
end;

end.
