unit uPaintBoard;

interface

uses
  Types,  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  DouglasPeuckers, sPanel, sComboBox, ComCtrls, ImgList,
  acAlphaImageList, ClipBrd,
  Buttons, Dialogs, ExtCtrls, sDialogs, sColorSelect, sTrackBar,
  sSpeedButton, uConst, uFunc, Inifiles, jpeg,
  sScrollBox,
  sSplitter, Math, StrUtils, sSkinProvider, GR32_Image, GR32, GR32_Layers, GR32_Resamplers,
  GR32_Filters, uPopUpWindow, uCommonGraphics, sHintManager,
  sLabel, ExtDlgs, StdCtrls, Menus, uDrawPictureSupport, Generics.Collections;

type
  TPictureData = record
    SenderName : String;
    PictureRect : TRect;
    MasterAlpha : Integer;
    TimeOut : DWORD;
  end;
  TDrawTimer = class (TTimer)
  private
    FLastExecutedTime : DWORD;
    procedure RestartIfNeed();
  public
    FX0 : Integer;
    FY0 : Integer;
    FX1 : Integer;
    FY1 : Integer;
    FPointArray : TArrayOfPoint;
    FShape : TShape;
    FPenWidth : Integer;
    FSegmentsToDrawCount : Integer;
    FText : String;
    FDrawCursor : Boolean;
    FFullRedraw : Boolean;
    procedure RestartTimer(AX0, AY0, AX1, AY1, APenWidth : Integer; AShape : TShape);
    procedure RestartTimerPenOrBrush(APointArray : TArrayOfPoint; AShape : TShape; AFullRedraw : Boolean = False);
    procedure RestartTimerText(AX0, AY0 : Integer; AText : String; ADrawCursor : Boolean);
    procedure StopEndExecuteIfNeed();
    constructor Create(AOwner: TComponent); override;
end;

type
  TfmPaintBoard = class(TForm)
    sPColors: TsPanel;
    sColorsPanel: TsColorsPanel;
    sPTools: TsPanel;
    sSBPen: TsSpeedButton;
    sTBPenSize: TsTrackBar;
    sSBRect: TsSpeedButton;
    sSBEllipse: TsSpeedButton;
    sSBLine: TsSpeedButton;
    sSBText: TsSpeedButton;
    sSBBorder: TsSpeedButton;
    sSBBorderAndFill: TsSpeedButton;
    sSBFill: TsSpeedButton;
    sTBFontSize: TsTrackBar;
    sCBFontName: TsComboBox;
    sSBBold: TsSpeedButton;
    sSBItalic: TsSpeedButton;
    sSBUnderline: TsSpeedButton;
    sSBSave: TsSpeedButton;
    sSavePictureDialog: TsSavePictureDialog;
    sSBCopy: TsSpeedButton;
    sSBClean: TsSpeedButton;
    sScrollBox: TsScrollBox;
    sTBAlphaChanel: TsTrackBar;
    sSBBucket: TsSpeedButton;
    sSBPicker: TsSpeedButton;
    sCSInticator: TsColorSelect;
    sSplitter: TsSplitter;
    sSkinProvider1: TsSkinProvider;
    iDrawArea: TImage32;
    sSBBrushStyle: TsSpeedButton;
    sSBPenColor: TsSpeedButton;
    sSBBrushColor: TsSpeedButton;
    sColorDialog: TsColorDialog;
    aiColors: TsAlphaImageList;
    sSBBrush: TsSpeedButton;
    sSBStamp: TsSpeedButton;
    sSBUndo: TsSpeedButton;
    sSBRedo: TsSpeedButton;
    lRedo: TBitmap32List;
    UpdateTimer: TTimer;
    sSBStampImage: TsSpeedButton;
    sHintManager: TsHintManager;
    pExpired: TsPanel;
    sWebLabel1: TsWebLabel;
    sSBLoadPictureFromFile: TsSpeedButton;
    sOpenPictureDialog: TsOpenPictureDialog;
    sSBPostPicture: TsSpeedButton;
    sSBCancelPicture: TsSpeedButton;
    sSBLoadPictureFromClipboard: TsSpeedButton;

    procedure FormCreate(Sender: TObject);
    procedure sColorsPanelChange(Sender: TObject);
    procedure sColorsPanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure DrawToolMouseClick(Sender: TObject);
    procedure OnKeyPress(Sender: TObject; var Key: Char);
    procedure OnShowForm(Sender: TObject);
    procedure TextAttrOnChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure sSBSaveClick(Sender: TObject);
    procedure sSBCopyClick(Sender: TObject);
    procedure pbDrawAreaClick(Sender: TObject);
    procedure sSBCleanClick(Sender: TObject);
    procedure sCSInticatorMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure sSplitterMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure sSplitterMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure sSplitterMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure iDrawAreaMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure iDrawAreaMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer; Layer: TCustomLayer);
    procedure iDrawAreaMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure sSBBrushStyleClick(Sender: TObject);
    procedure sSBPenColorClick(Sender: TObject);
    procedure sTBPenSizeChange(Sender: TObject);
    procedure sSBUndoClick(Sender: TObject);
    procedure sSBRedoClick(Sender: TObject);
    procedure UpdateTimerTimer(Sender: TObject);
    procedure sSBStampImageClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure sSplitterDblClick(Sender: TObject);
    procedure sTBAlphaChanelChange(Sender: TObject);
    procedure sSBLoadPictureFromFileClick(Sender: TObject);
    procedure sSBPostPictureClick(Sender: TObject);
    procedure sSBCancelPictureClick(Sender: TObject);
    procedure sSBLoadPictureFromClipboardClick(Sender: TObject);
  private
    FIsPrivateChannel : Boolean;
    FNowDrawing: TShape;
    FPostedLayersCount : Integer;
    FLastSelectedShape : TShape;
    X0, Y0: Integer;
    FPenDown: Boolean;
    FOrigList: TArrayOfPoint;
    FLastMouseButton : TMouseButton;
    FTextToDraw : String;
    FReadyEnterText : Boolean;
    FMouseButtonDrawTextDown : TMouseButton;
    FParentChanName: string;
    FParentChanHandle: HWND;
    FBoardResizing: Boolean;
    FStartResizingYCoord: Integer;
    FStartResizingHeight: Integer;
    FMaxResizingHeigth: Integer;
    FChatWindowHeight: Integer;
    FMaskBitmap : TBitmap32;
    FCurrentBrush: TMyBitmap32;
    FCurrentPen : TMyBitmap32;
    FCurrentBrushTexture: TTexture;
    FCurrentPenTexture: TTexture;
    FInvertTextures : Boolean;
    FDrawTimer : TDrawTimer;
    FCurrentStamp: TMyBitmap32;
    FPluginIsExpired: Boolean;
    FInsertPictureLayer : TPictureLayer;
    FAllowImageInserting : Boolean;
    FPictureDataBuffer : TDictionary<String, TPictureData>;
    procedure OnDrawTimer(Sender: TObject);

    procedure ReDrawTextOnDrawLayer(ADrawCursor : Boolean = True);
    procedure SendDataPen();
    procedure SendDataStamp();
    procedure SendDataRect();
    procedure SendDataText(Font : TFont; Text : String; X, Y : Integer);
    procedure SendDataBucket(X, Y : Integer);
    procedure SendDataClear();
    procedure SendDataUndo(IsUndo : Boolean);

    procedure AddPointToCurve(X, Y: integer);
    procedure SetParentChanName(const Value: string);
    procedure SetParentChanHandle(const Value: HWND);
    procedure SetBoardResizing(const Value: Boolean);
    procedure SetStartResizingYCoord(const Value: Integer);
    procedure SetStartResizingHeight(const Value: Integer);
    procedure SetMaxResizingHeigth(const Value: Integer);
    procedure SetChatWindowHeight(const Value: Integer);
    function GetDrawLayerBitmap: TBitmap32;
    procedure SetCurrentTextureOrColor(AIsPenTexture: Boolean; ATextureIndex: Integer; AColor: TColor = clBlack);
    function GetCurrentBrushTexture: TTexture;
    function GetCurrentPenTexture: TTexture;
    procedure SetPluginIsExpired(const Value: Boolean);

    property BoardResizing : Boolean read FBoardResizing write SetBoardResizing;
    property StartResizingYCoord : Integer read FStartResizingYCoord write SetStartResizingYCoord;
    property StartResizingHeight : Integer read FStartResizingHeight write SetStartResizingHeight;
    property MaxResizingHeigth : Integer read FMaxResizingHeigth write SetMaxResizingHeigth;
    function CurrentAlphaValue() : Integer;
    procedure PrepareDrawArea();
    property DrawLayerBitmap : TBitmap32 read GetDrawLayerBitmap;
    property CurrentBrush : TMyBitmap32 read FCurrentBrush write FCurrentBrush;
    property CurrentPen : TMyBitmap32 read FCurrentPen write FCurrentPen;
    property CurrentStamp : TMyBitmap32 read FCurrentStamp write FCurrentStamp;
    property CurrentPenTexture : TTexture read GetCurrentPenTexture;
    property CurrentBrushTexture : TTexture read GetCurrentBrushTexture;
    procedure SetBrush(ABrushBitmap : TMyBitmap32; ABrushIndex : Integer; AChangeIcon : Boolean = True);
    procedure SetStampImage(AStampImage : TMyBitmap32; AStampImageIndex : Integer; AChangeIcon : Boolean = True);
    procedure SetPen(APenBitmap : TMyBitmap32; APenSize : Integer);
    function CanUndo() : Boolean;
    function CanRedo() : Boolean;
    procedure Undo(ASendData : Boolean = True);
    procedure Redo(ASendData : Boolean = True);
    procedure ClearRedoStack();
    procedure PostDrawLayer();
    procedure PostRecivedLayer(ARecivedBitmap : TBitmap32);
    procedure SetFocusSafe(AControl : TWinControl);
    function GetFigureFilling(): TFigureFilling;
    function GetParentPanelSize(): TSize;
    procedure PlaceControls();
  public
    procedure SetFocusToBoard();
    property ParentChanName : string read FParentChanName write SetParentChanName;
    property ChatWindowHeight : Integer read FChatWindowHeight write SetChatWindowHeight;

    property ParentChanHandle : HWND read FParentChanHandle write SetParentChanHandle;
    procedure FixWindowPos();
    function DecodeDataAsPictureData(ASenderName : String; Data : String): Boolean;
    procedure DrawPictureIfNeed(ASenderName : String; APictureStream : TStream);
    procedure DecodeInputString(Data : String);
    property PluginIsExpired : Boolean read FPluginIsExpired write SetPluginIsExpired;
  end;

implementation
{$R *.dfm}

const DRAW_LAYER_INDEX = 11;
      RECIVE_LAYER_INDEX = DRAW_LAYER_INDEX - 1;

//--------------------------------------------------------------------------------------------------
function TfmPaintBoard.GetParentPanelSize: TSize;
var WindowRect: TRect;
begin
  if GetWindowRect(Self.ParentWindow, WindowRect) then
  begin
    Result.cx := WindowRect.Right - WindowRect.Left;
    Result.cy := WindowRect.Bottom - WindowRect.Top;
  end
  else
  begin
    Result.cx := 0;
    Result.cy := 0;
  end;
end;

procedure TfmPaintBoard.FixWindowPos;
var ChatPanelSize : TSize;
begin
  if (not BoardResizing) and Visible then
    begin
      ChatPanelSize := GetParentPanelSize();
      Height := Max(sSplitter.Height, ChatPanelSize.cy - ChatWindowHeight);
      Width := ChatPanelSize.cx;
      MoveWindow(ParentChanHandle, 0, Height, Width, ChatPanelSize.cy - Height, True);
    end;
end;

procedure TfmPaintBoard.FormCreate(Sender: TObject);
begin
  Inherited;
  FIsPrivateChannel := False;
  FReadyEnterText := False;
  FParentChanHandle := 0;
  SetLength(FParentChanName, 0);
  BorderStyle := bsNone;
  FPostedLayersCount := 0;
  PrepareDrawArea();

  PluginIsExpired := False;
  sWebLabel1.Caption := Format(sWebLabel1.Caption, [CHAT_NAME]);
  FInvertTextures := False;
  FMaskBitmap := TBitmap32.Create();
  FMaskBitmap.SetSizeFrom(iDrawArea);
  FCurrentBrush := TMyBitmap32.Create();
  FCurrentStamp := TMyBitmap32.Create();
  FCurrentStamp.Resampler := TLinearResampler.Create();
  FCurrentPen := TMyBitmap32.Create();
  FCurrentPenTexture := TTexture.Create();
  FCurrentBrushTexture := TTexture.Create();
  SetBrush(FCurrentBrush, 0);
  SetStampImage(FCurrentStamp, 0);
  SetPen(FCurrentPen, sTBPenSize.Position);
  SetCurrentTextureOrColor(True, 0);
  SetCurrentTextureOrColor(False, 1);


  FDrawTimer := TDrawTimer.Create(nil);
  FDrawTimer.OnTimer := OnDrawTimer;

  FNowDrawing := sPen;
  FLastSelectedShape := sPen;
  FPenDown := False;
  sCBFontName.Items.Text := 'Arial'#13#10'Comic Sans MS'#13#10'Courier New'#13#10'Garamond'#13#10'Microsoft Sans Serif'#13#10+
                   'Tahoma'#13#10'Times New Roman'#13#10'Trebuchet MS'#13#10'Verdana';

  sCBFontName.ItemIndex := 0;

  sPTools.Height := 26;
  sPColors.Height := 28;

  Screen.Cursors[crPicker]    := LoadCursor(hInstance,'PICKER');
  Screen.Cursors[crBucket]    := LoadCursor(hInstance,'BUCKET');
  Screen.Cursors[crRectangle] := LoadCursor(hInstance,'RECTANGLE');
  Screen.Cursors[crEllipse]   := LoadCursor(hInstance,'ELLIPSE');
  Screen.Cursors[crBrush]     := LoadCursor(hInstance,'BRUSH');
  Screen.Cursors[crLine]      := LoadCursor(hInstance,'LINE');
  Screen.Cursors[crPencil]    := LoadCursor(hInstance,'PENCIL');
  Screen.Cursors[crStamp]     := LoadCursor(hInstance,'STAMP');

  Visible := False;
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.FormDestroy(Sender: TObject);
var CPSize : TSize;
    TmpInt : Integer;
begin
  try
    if IsWindow(ParentChanHandle) then
      begin
        CPSize := GetParentPanelSize();
        MoveWindow(ParentChanHandle, 0, 0, CPSize.cx, CPSize.cy, True);
      end;
    MemIniFile.WriteInteger(ParentChanName, 'ChatWindowHeight', ChatWindowHeight);
    MemIniFile.WriteInteger(ParentChanName, 'CurrentFont', sCBFontName.ItemIndex);
    MemIniFile.WriteInteger(ParentChanName, 'FontSize', sTBFontSize.Position);
    MemIniFile.WriteInteger(ParentChanName, 'PenSize', sTBPenSize.Position);
    MemIniFile.WriteInteger(ParentChanName, 'Transparent', sTBAlphaChanel.Position);

    MemIniFile.WriteBool(ParentChanName, 'FontBold', sSBBold.Down);
    MemIniFile.WriteBool(ParentChanName, 'FontItalic', sSBItalic.Down);
    MemIniFile.WriteBool(ParentChanName, 'FontUnderline', sSBUnderline.Down);

    MemIniFile.WriteInteger(ParentChanName, 'PenTextureColor', WinColor(FCurrentPenTexture.TextureColor));
    MemIniFile.WriteInteger(ParentChanName, 'PenTextureIndex', FCurrentPenTexture.TextureBitmapIndex);

    MemIniFile.WriteInteger(ParentChanName, 'BrushTextureColor', WinColor(FCurrentBrushTexture.TextureColor));
    MemIniFile.WriteInteger(ParentChanName, 'BrushTextureIndex', FCurrentBrushTexture.TextureBitmapIndex);

    MemIniFile.WriteInteger(ParentChanName, 'BrushStyleIndex', CurrentBrush.Tag);
    MemIniFile.WriteInteger(ParentChanName, 'StampImageIndex', CurrentStamp.Tag);

    MemIniFile.WriteInteger(ParentChanName, 'BrushTextureIndex', FCurrentBrushTexture.TextureBitmapIndex);

    MemIniFile.WriteString(GEN_SET_SECTION, 'CustomColors', StringReplace(sColorDialog.CustomColors.Text, #13#10, '|', [rfReplaceAll]));

    TmpInt :=
      Integer(sSBPen.Down)     * 1  +
      Integer(sSBBrush.Down)   * 2  +
      Integer(sSBStamp.Down)   * 3 +
      Integer(sSBBucket.Down)  * 4 +
      Integer(sSBPicker.Down)  * 5 +
      Integer(sSBRect.Down)    * 6 +
      Integer(sSBEllipse.Down) * 7 +
      Integer(sSBLine.Down)    * 8 +
      Integer(sSBText.Down)    * 9;

    MemIniFile.WriteInteger(ParentChanName, 'CurrentShape', TmpInt);

    TmpInt :=
      Integer(sSBBorder.Down) * 1 +
      Integer(sSBFill.Down) * 2 +
      Integer(sSBBorderAndFill.Down) * 3;
    MemIniFile.WriteInteger(ParentChanName, 'ShapeStyle', TmpInt);
    MemIniFile.UpdateFile();
  finally
    FreeAndNilA(FMaskBitmap);
    FreeAndNilA(FCurrentBrush);
    FreeAndNilA(FCurrentStamp);
    FreeAndNilA(FCurrentPen);
    FreeAndNilA(FCurrentBrushTexture);
    FreeAndNilA(FCurrentPenTexture);
    FreeAndNilA(FDrawTimer);
    FreeAndNilA(FInsertPictureLayer);
    FreeAndNilA(FPictureDataBuffer);
  end;
  Inherited;
end;

procedure TfmPaintBoard.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if FPluginIsExpired or Assigned(FInsertPictureLayer) then
    Exit;
  if (ssCtrl in Shift) and (Key = Ord('Z')) then
  begin
    Undo();
    SetFocusSafe(iDrawArea);
  end;
  if (ssCtrl in Shift) and (Key = Ord('Y')) then
  begin
    Redo();
    SetFocusSafe(iDrawArea);
  end;
  if (ssCtrl in Shift) and (Key = Ord('C')) then
  begin
    sSBCopyClick(sSBCopy);
    SetFocusSafe(iDrawArea);
  end;
  if (ssCtrl in Shift) and (Key = Ord('S')) then
  begin
    sSBSaveClick(sSBSave);
    SetFocusSafe(iDrawArea);
  end;
  if (ssCtrl in Shift) and (Key = Ord('V')) then
  begin
    sSBLoadPictureFromClipboardClick(sSBLoadPictureFromClipboard);
    SetFocusSafe(iDrawArea);
  end;
  if (ssCtrl in Shift) and (Key = Ord('O')) then
  begin
    sSBLoadPictureFromFileClick(sSBLoadPictureFromFile);
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.FormResize(Sender: TObject);
var ShowPanels : Boolean;
begin
  ShowPanels := Height >= (sPColors.Height + sPTools.Height + sSplitter.Height);
  sPTools.Visible := ShowPanels and (not FPluginIsExpired);
  sPColors.Visible := ShowPanels and (not FPluginIsExpired);
end;


function TfmPaintBoard.GetCurrentBrushTexture: TTexture;
begin
 if not FInvertTextures then
   Result := FCurrentBrushTexture
 else
   Result := FCurrentPenTexture;
end;

function TfmPaintBoard.GetCurrentPenTexture: TTexture;
begin
 if not FInvertTextures then
   Result := FCurrentPenTexture
 else
   Result := FCurrentBrushTexture;
end;

function TfmPaintBoard.GetDrawLayerBitmap: TBitmap32;
begin
  Result := (iDrawArea.Layers[DRAW_LAYER_INDEX] as TBitmapLayer).Bitmap;
end;

function TfmPaintBoard.GetFigureFilling: TFigureFilling;
begin
  if sSBBorderAndFill.Down then
    Result := ffBorderAndFill
  else
  if sSBBorder.Down then
    Result := ffBorder
  else
    Result := ffFill;
end;

procedure TfmPaintBoard.iDrawAreaMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
begin
  if FPluginIsExpired or Assigned(FInsertPictureLayer) then
    Exit;

  SetFocusSafe(iDrawArea);
  if sSBPicker.Down or (Button = mbMiddle) then
    Exit;

  FInvertTextures := Button = mbRight;

  if sSBBucket.Down then
    begin
      dmCommonGraphics.FloodFill(X, Y, 20, iDrawArea, DrawLayerBitmap, CurrentPenTexture);
      PostDrawLayer();
      SendDataBucket(X, Y);
      Exit;
    end;
  //          
  if FPenDown then
  begin
    FPenDown := False;
    FDrawTimer.Enabled := False;
    DrawLayerBitmap.Clear(0);
    FMaskBitmap.Clear();
    SetLength(FOrigList, 0);
  end
  else
  begin
    //                
    if (Button = mbRight) and not(sSBText.Down) then
      begin
        SetCaptureControl((Sender as TControl));
      end;
    X0 := X;
    Y0 := Y;
    FPenDown := True;
    if sSBPen.Down then
      begin
        FNowDrawing := sPen;
        SetLength(FOrigList, 0);
        AddPointToCurve(X, Y);
        FDrawTimer.RestartTimerPenOrBrush(FOrigList, FNowDrawing);
      end;
    if sSBBrush.Down then
      begin
        FNowDrawing := sBrush;
        SetLength(FOrigList, 0);
        AddPointToCurve(X, Y);
        FDrawTimer.RestartTimerPenOrBrush(FOrigList, FNowDrawing);
      end;
    if sSBStamp.Down then
      begin
        FNowDrawing := sStamp;
        FDrawTimer.RestartTimer(X, Y, 0, 0, sTBPenSize.Position, FNowDrawing);
      end;
    if sSBEllipse.Down or sSBRect.Down then
      begin
        if sSBEllipse.Down then
          FNowDrawing := sEllipse
        else
          FNowDrawing := sRect;
      end;
    if sSBLine.Down then
      begin
        FNowDrawing := sLine;
      end;
    if sSBText.Down then
      begin
        FPenDown := False;
        FNowDrawing := sText;
        FMouseButtonDrawTextDown := Button;
        X0 := X;
        FReadyEnterText := True;
        TextAttrOnChange(nil);
        Y0 := Y - Round(DrawLayerBitmap.TextHeight('_|') / 2); //  TextAttrOnChange   
        ReDrawTextOnDrawLayer();
      end;
  end;
end;

procedure TfmPaintBoard.iDrawAreaMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer; Layer: TCustomLayer);
begin
  if FPluginIsExpired or Assigned(FInsertPictureLayer) then
    Exit;

  if sSBPicker.Down then
    sCSInticator.ColorValue := iDrawArea.Canvas.Pixels[X, Y];
  if not FPenDown then
    Exit;
  case FNowDrawing of
    sPen, sBrush:
      begin
        AddPointToCurve(X, Y);
        FDrawTimer.RestartTimerPenOrBrush(FOrigList, FNowDrawing);
      end;
    sRect, sEllipse:
      begin
        FDrawTimer.RestartTimer(X0, Y0, X, Y, sTBPenSize.Position, FNowDrawing);
      end;
    sStamp:
      begin
        FDrawTimer.RestartTimer(X, Y, 0, 0, sTBPenSize.Position, FNowDrawing);
      end;
    sLine:
      begin
        FDrawTimer.RestartTimer(X0, Y0, X, Y, 0, FNowDrawing);
      end;
    sText:
      begin
      end;
  end;
end;

procedure TfmPaintBoard.iDrawAreaMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
var
  ALength: integer;
begin
  if FPluginIsExpired or Assigned(FInsertPictureLayer) then
    Exit;
  try
    if (Button = mbMiddle) then
      Exit;

    if (Button = mbRight) then
      ReleaseCapture();

    if sSBPicker.Down then
      begin
        SetCurrentTextureOrColor(Button = mbLeft, -1, iDrawArea.Canvas.Pixels[X, Y]);
        //   
        case FLastSelectedShape  of
          sPen     : sSBPen.Down := True;
          sRect    : sSBRect.Down := True;
          sEllipse : sSBEllipse.Down := True;
          sText    : sSBText.Down := True;
          sLine    : sSBLine.Down := True;
          sBucket  : sSBBucket.Down := True;
        end;
        DrawToolMouseClick(nil);
        Exit;
      end;

    if FPenDown then
      begin
        FPenDown := False;
        If (FNowDrawing = sPen) or (FNowDrawing = sBrush) then
          begin
            //     
            if length(FOrigList) > 2 then
              begin
//                AddLogStr(IntToStr(length(FOrigList)) + ' do');
                ALength := PolySimplifyArray(0.7, FOrigList);
//                AddLogStr(IntToStr(ALength) + ' posle');
                SetLength(FOrigList, ALength);
              end;
            //     
            FDrawTimer.RestartTimerPenOrBrush(FOrigList, FNowDrawing, True);
            FDrawTimer.StopEndExecuteIfNeed();
            //   
            SendDataPen();
            PostDrawLayer();
            SetLength(FOrigList, 0);
            Exit;
          end;
        //       
        FDrawTimer.StopEndExecuteIfNeed();
        if FNowDrawing = sLine then
          begin
            PostDrawLayer();
            SendDataPen();
          end;
        if (FNowDrawing = sRect) or (FNowDrawing = sEllipse) then
          begin
            PostDrawLayer();
            SendDataRect();
          end;
        if (FNowDrawing = sStamp) then
          begin
            PostDrawLayer();
            SendDataStamp();
          end;
      end;
    if (FNowDrawing = sText) and sTBFontSize.Visible then
      begin
        SetFocusSafe(sTBFontSize);
      end;

  except
     on E : Exception do
       AddLogStr(E.Message, 'TfmPaintBoard.iDrawAreaMouseUp()');
  end;
end;

procedure TfmPaintBoard.PostDrawLayer();
var ALayer : TBitmapLayer;
    i : integer;
    TmpBitmap : TBitmap32;
begin
  ALayer := (iDrawArea.Layers[0] as TBitmapLayer);
  TmpBitmap := TBitmap32.Create();
  try
    TmpBitmap.SetSizeFrom(iDrawArea.Bitmap);
    for i := 1 to iDrawArea.Layers.Count - 1 do
      iDrawArea.Layers[i].Visible := False;
    try
      iDrawArea.PaintTo(TmpBitmap, iDrawArea.GetBitmapRect);
      iDrawArea.Bitmap.Assign(TmpBitmap);
      //      
      ALayer.Bitmap.Clear(clBlack);
      ALayer.Index := DRAW_LAYER_INDEX;
      Inc(FPostedLayersCount);
      ClearRedoStack();
      FMaskBitmap.Clear(clBlack32);
    finally
      for i := 0 to iDrawArea.Layers.Count - 1 do
        iDrawArea.Layers[i].Visible := True;
    end;
  finally
    TmpBitmap.Free();
  end;
  DrawLayerBitmap.MasterAlpha := CurrentAlphaValue();
end;

procedure TfmPaintBoard.PostRecivedLayer(ARecivedBitmap : TBitmap32);
var ALayer : TBitmapLayer;
    i : integer;
    TmpBitmap : TBitmap32;
begin
  ALayer := (iDrawArea.Layers[0] as TBitmapLayer);
  TmpBitmap := TBitmap32.Create();
  try
    TmpBitmap.SetSizeFrom(iDrawArea.Bitmap);
    for i := 1 to iDrawArea.Layers.Count - 1 do
      iDrawArea.Layers[i].Visible := False;
    try
      iDrawArea.PaintTo(TmpBitmap, iDrawArea.GetBitmapRect);
      iDrawArea.Bitmap.Assign(TmpBitmap);
      //      (   )
      ALayer.Bitmap := ARecivedBitmap;
      ALayer.Index := RECIVE_LAYER_INDEX;
      Inc(FPostedLayersCount);
      ClearRedoStack();
    finally
      for i := 0 to iDrawArea.Layers.Count - 1 do
        iDrawArea.Layers[i].Visible := True;
    end;
  finally
    TmpBitmap.Free();
  end;
end;


procedure TfmPaintBoard.PrepareDrawArea();
var i : Integer;
    ALayer : TBitmapLayer;
    L : TFloatRect;
begin
  iDrawArea.Left := 0;
  iDrawArea.Top := 0;
  iDrawArea.Width := 800;
  iDrawArea.Height := 600;
  iDrawArea.Layers.Clear();
  iDrawArea.BeginUpdate;
  try
    for i := 0 to DRAW_LAYER_INDEX do
    begin
      // create a new layer...
      ALayer := TBitmapLayer.Create(iDrawArea.Layers);
      with ALayer do
      begin
        Bitmap := TBitmap32.Create();
        Bitmap.DrawMode := dmBlend;
        Bitmap.SetSize(iDrawArea.Width, iDrawArea.Height);
        Bitmap.MasterAlpha := $FF;
        Bitmap.Clear(0); // 
        L.Left := 0;
        L.Top := 0;
        L.Right := Bitmap.Width;
        L.Bottom := Bitmap.Height;
        ALayer.Location := L;
      end;
    end;
    iDrawArea.Bitmap := TBitmap32.Create();
    iDrawArea.Bitmap.SetSizeFrom(iDrawArea);
    iDrawArea.Bitmap.Clear(clWhite32);
  finally
    iDrawArea.EndUpdate;
    iDrawArea.Changed;
  end;
  DrawLayerBitmap.MasterAlpha := CurrentAlphaValue();
end;

//--------------------------------------------------------------------------------------------------
//1  -  P B +    + 2   + 2      +  
procedure TfmPaintBoard.SendDataPen();
var DataString : String;
    i : Integer;
begin
  case FDrawTimer.FShape of
    sLine  : begin
               DataString := 'P' + IntToHex(CurrentAlphaValue(), 2) +
                 CurrentPenTexture.EncodeTexture + IntToHex(CurrentPen.Tag, 2) +
                 IntToStr32x(FDrawTimer.FX0) + IntToStr32x(FDrawTimer.FY0) +
                 IntToStr32x(FDrawTimer.FX1) + IntToStr32x(FDrawTimer.FY1);
             end;
    sPen   : begin
               DataString := 'P' + IntToHex(CurrentAlphaValue(), 2) +
                 CurrentPenTexture.EncodeTexture + IntToHex(CurrentPen.Tag, 2);
               For i := 0 To Length(FDrawTimer.FPointArray) - 1 do
                 DataString := DataString +
                               IntToStr32x(FDrawTimer.FPointArray[i].X) +
                               IntToStr32x(FDrawTimer.FPointArray[i].Y);
             end;
    sBrush : begin
               DataString := 'B' + IntToHex(CurrentAlphaValue(), 2) +
                 CurrentPenTexture.EncodeTexture + IntToHex(CurrentBrush.Tag, 2);
               For i := 0 To Length(FDrawTimer.FPointArray) - 1 do
                 DataString := DataString +
                               IntToStr32x(FDrawTimer.FPointArray[i].X) +
                               IntToStr32x(FDrawTimer.FPointArray[i].Y);
             end;
  else
    Exit;
  end;
  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.SendDataRect();
var DataString : String;
begin
  case FDrawTimer.FShape of
    sRect    : DataString := 'R';
    sEllipse : DataString := 'E';
  else
    Exit;
  end;
  DataString := DataString + IntToHex(CurrentAlphaValue(), 2) +
    CurrentPenTexture.EncodeTexture + CurrentBrushTexture.EncodeTexture +
    IntToStr(Integer(GetFigureFilling())) + IntToHex(CurrentPen.Tag, 2) +
    IntToStr32x(FDrawTimer.FX0) + IntToStr32x(FDrawTimer.FY0) +
    IntToStr32x(FDrawTimer.FX1) + IntToStr32x(FDrawTimer.FY1);

  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;

procedure TfmPaintBoard.SendDataStamp;
var DataString : String;
begin
  DataString := 'S' + IntToHex(CurrentAlphaValue(), 2) + CurrentPenTexture.EncodeTexture() +
   IntToHex(FDrawTimer.FPenWidth, 2) + IntToHex(CurrentStamp.Tag, 2) +
   IntToStr32x(FDrawTimer.FX0) + IntToStr32x(FDrawTimer.FY0);
  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;

//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.SendDataBucket(X, Y : Integer);
var DataString : String;
begin
  DataString := 'F' + IntToHex(CurrentAlphaValue(), 2) +
    CurrentPenTexture.EncodeTexture + IntToStr32x(X) + IntToStr32x(Y);
  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.SendDataClear();
var DataString : String;
begin
  DataString := 'CLBRD';
  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.SendDataText(Font : TFont; Text : String; X, Y : Integer);
var DataString : String;
    FS : Integer;
begin
  If Length(Trim(Text)) = 0 then
    Exit;
  DataString := 'T';
  FS := 0;
  if fsBold in Font.Style then
    FS := 1;
  if fsItalic in Font.Style then
    Inc(FS,2);
  if fsUnderline in Font.Style then
    Inc(FS,4);

  DataString := DataString + IntToHex(CurrentAlphaValue(), 2) +
    CurrentPenTexture.EncodeTexture() + IntToStr(FS) +
    IntToStr(sCBFontName.Items.IndexOf(Font.Name)) + IntToHex(Font.Size, 2) +
    IntToStr32x(X) + IntToStr32x(Y) + ' ' + Text + ' ';
  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;

procedure TfmPaintBoard.SendDataUndo(IsUndo: Boolean);
var DataString : String;
begin
  if IsUndo then
    DataString := 'UNDO'
  else
    DataString := 'REDO';
  SendPaintData(ParentChanName, DataString);
  SetFocusSafe(iDrawArea);
end;

procedure TfmPaintBoard.SetBoardResizing(const Value: Boolean);
begin
  FBoardResizing := Value;
end;

procedure TfmPaintBoard.SetBrush(ABrushBitmap: TMyBitmap32; ABrushIndex: Integer; AChangeIcon : Boolean = True);
var TmpBitmap : TBitmap32;
begin
  TmpBitmap := dmCommonGraphics.GetBrush(ABrushIndex);
  if not Assigned(TmpBitmap) then
    Exit;
  try
    ABrushBitmap.BeginUpdate();
    try
      ABrushBitmap.SetSizeFrom(TmpBitmap);
      ABrushBitmap.Clear(clWhite32);
      IntensityToAlpha(ABrushBitmap, TmpBitmap);
      ABrushBitmap.Tag := ABrushIndex;
    finally
      ABrushBitmap.EndUpdate();
      ABrushBitmap.Changed();
    end;
    if AChangeIcon then
      sSBBrushStyle.ImageIndex := ABrushIndex;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TfmPaintBoard.SetBrush()');
  end;
end;

procedure TfmPaintBoard.SetPen(APenBitmap: TMyBitmap32; APenSize : Integer);
var TmpBitmap : TBitmap32;
begin
  if (APenSize <= 0) then
    Exit;
  TmpBitmap := dmCommonGraphics.lPen.Bitmap[0];
  try
    APenBitmap.BeginUpdate();
    try
      if APenSize = 1 then
      begin
        APenBitmap.SetSize(1,1);
        APenBitmap.Clear(clWhite32);
      end
      else
      begin
        APenBitmap.SetSizeFrom(TmpBitmap);
        APenBitmap.Clear(clWhite32);
        IntensityToAlpha(APenBitmap, TmpBitmap);
        dmCommonGraphics.Resize(APenBitmap, APenSize);
      end;
      APenBitmap.Tag := APenSize;
    finally
      APenBitmap.EndUpdate();
      APenBitmap.Changed();
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TfmPaintBoard.SetPen()');
  end;
end;

procedure TfmPaintBoard.SetPluginIsExpired(const Value: Boolean);
begin
  FPluginIsExpired := Value;
  sPTools.Visible    := not FPluginIsExpired;
  iDrawArea.Visible := not FPluginIsExpired;
  sPColors.Visible   := not FPluginIsExpired;
  pExpired.Visible   := FPluginIsExpired;
  if FPluginIsExpired then
    begin
      pExpired.Align := alClient;
    end;
  sWebLabel1.URL := URL_FOR_PLUGIN_UPDATE;
end;

procedure TfmPaintBoard.SetChatWindowHeight(const Value: Integer);
begin
  FChatWindowHeight := Value;
end;

procedure TfmPaintBoard.SetFocusSafe(AControl: TWinControl);
begin
  try
    if (AControl.Visible) and (AControl.CanFocus) then
      AControl.SetFocus();
  except

  end;
end;

procedure TfmPaintBoard.SetFocusToBoard();
begin
  SetFocusSafe(iDrawArea);
end;

procedure TfmPaintBoard.SetMaxResizingHeigth(const Value: Integer);
begin
  FMaxResizingHeigth := Value;
end;

procedure TfmPaintBoard.SetStampImage(AStampImage: TMyBitmap32;
  AStampImageIndex: Integer; AChangeIcon: Boolean);
var TmpBitmap : TBitmap32;
begin
  try
    TmpBitmap := dmCommonGraphics.GetStamp(AStampImageIndex);
    if Assigned(TmpBitmap) then
      AStampImage.Assign(TmpBitmap)
    else
      Exit;
    AStampImage.Tag := AStampImageIndex;
    if AChangeIcon then
      sSBStampImage.ImageIndex := AStampImageIndex;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TfmPaintBoard.SetStampImage()');
  end;
end;

procedure TfmPaintBoard.SetStartResizingHeight(const Value: Integer);
begin
  FStartResizingHeight := Value;
end;

procedure TfmPaintBoard.SetStartResizingYCoord(const Value: Integer);
begin
  FStartResizingYCoord := Value;
end;

procedure TfmPaintBoard.SetCurrentTextureOrColor(AIsPenTexture: Boolean;
  ATextureIndex: Integer; AColor: TColor = clBlack);
var TextureButton : TsSpeedButton;
    CurrentTexture : TTexture;
    TmpBitmap : TBitmap;
begin
  if AIsPenTexture then
    begin
      CurrentTexture := FCurrentPenTexture;
      TextureButton := sSBPenColor;
    end
  else
    begin
      CurrentTexture := FCurrentBrushTexture;
      TextureButton := sSBBrushColor;
    end;
  if ATextureIndex >= 0 then
    begin
      CurrentTexture.TextureBitmapIndex := ATextureIndex;
      TextureButton.Images := dmCommonGraphics.aiTextures;
      TextureButton.ImageIndex  := ATextureIndex;
    end
  else
    begin
      CurrentTexture.TextureColor := Color32(AColor);
      TmpBitmap := TBitmap.Create();
      try
        TmpBitmap.Width := 24;
        TmpBitmap.Height := 24;
        TmpBitmap.Canvas.Brush.Color := $101578;
        TmpBitmap.Canvas.Rectangle(-2, -2, TmpBitmap.Width + 2, TmpBitmap.Height + 2);
        TmpBitmap.Canvas.Brush.Color := AColor;
        TmpBitmap.Canvas.Pen.Color := AColor;
        TmpBitmap.Canvas.RoundRect(0, 0, TmpBitmap.Width, TmpBitmap.Height, 6, 6);
        aiColors.ReplaceMasked(Integer(AIsPenTexture), TmpBitmap, $101578);
      finally
        FreeAndNil(TmpBitmap);
      end;
      TextureButton.Images := aiColors;
      TextureButton.ImageIndex := -1;
      TextureButton.ImageIndex := Integer(AIsPenTexture);
    end;
  TextAttrOnChange(sTBFontSize);  //     
end;

procedure TfmPaintBoard.SetParentChanHandle(const Value: HWND);
begin
  FParentChanHandle := Value;
end;

procedure TfmPaintBoard.SetParentChanName(const Value: string);
var TmpColor : TColor;
    TmpIndex : Integer;
begin
  FParentChanName := Value;
  FIsPrivateChannel := (Length(FParentChanName) > 0) and (FParentChanName[1] = '&');
  ChatWindowHeight := MemIniFile.ReadInteger(ParentChanName, 'ChatWindowHeight',50);
  sCBFontName.ItemIndex := Max(0, MemIniFile.ReadInteger(ParentChanName, 'CurrentFont', 0));
  sTBFontSize.Position := MemIniFile.ReadInteger(ParentChanName, 'FontSize', 5);
  sTBPenSize.Position := MemIniFile.ReadInteger(ParentChanName, 'PenSize', 10);
  sTBAlphaChanel.Position := MemIniFile.ReadInteger(ParentChanName, 'Transparent', 0);

  TmpColor := MemIniFile.ReadInteger(ParentChanName, 'PenTextureColor', $00FF8000);
  TmpIndex := MemIniFile.ReadInteger(ParentChanName, 'PenTextureIndex', 0);
  SetCurrentTextureOrColor(True, TmpIndex, TmpColor);

  TmpColor := MemIniFile.ReadInteger(ParentChanName, 'BrushTextureColor', $0000A0FF);
  TmpIndex := MemIniFile.ReadInteger(ParentChanName, 'BrushTextureIndex', -1);
  SetCurrentTextureOrColor(False, TmpIndex, TmpColor);

  sSBBold.Down := MemIniFile.ReadBool(ParentChanName, 'FontBold', False);
  sSBItalic.Down := MemIniFile.ReadBool(ParentChanName, 'FontItalic', False);
  sSBUnderline.Down := MemIniFile.ReadBool(ParentChanName, 'FontUnderline', False);

  TmpIndex := MemIniFile.ReadInteger(ParentChanName, 'BrushStyleIndex', 0);
  SetBrush(CurrentBrush, TmpIndex);

  TmpIndex := MemIniFile.ReadInteger(ParentChanName, 'StampImageIndex', 0);
  SetStampImage(CurrentStamp, TmpIndex);

  sColorDialog.CustomColors.Text := StringReplace(MemIniFile.ReadString(GEN_SET_SECTION, 'CustomColors', ''), '|', #13#10, [rfReplaceAll]);

  case MemIniFile.ReadInteger(ParentChanName, 'CurrentShape', 1) of
   1 : sSBPen.Down := True;
   2 : sSBBrush.Down := True;
   3 : sSBStamp.Down := True;
   4 : sSBBucket.Down := True;
   5 : sSBPicker.Down := True;
   6 : sSBRect.Down := True;
   7 : sSBEllipse.Down := True;
   8 : sSBLine.Down := True;
   9 : sSBText.Down := True;
  end;

  case MemIniFile.ReadInteger(ParentChanName, 'ShapeStyle', 1) of
   1 : sSBBorder.Down := True;
   2 : sSBFill.Down := True;
   3 : sSBBorderAndFill.Down := True;
  end;

  FAllowImageInserting := FIsPrivateChannel or (GetImageLimitation(FParentChanName) > 0);
  PlaceControls();
end;


//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.sColorsPanelChange(Sender: TObject);
begin
  if sColorsPanel.ItemIndex <> -1 then
    begin
      if FLastMouseButton = mbLeft then
        begin
          SetCurrentTextureOrColor(True, -1, sColorsPanel.ColorValue);
        end;
      if FLastMouseButton = mbRight then
        SetCurrentTextureOrColor(False, -1, sColorsPanel.ColorValue);
      sColorsPanel.ItemIndex := -1;
   end;
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.sColorsPanelMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FLastMouseButton := Button;
end;

procedure TfmPaintBoard.sCSInticatorMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Abort();
end;

procedure TfmPaintBoard.DrawPictureIfNeed(ASenderName : String; APictureStream: TStream);
var PictureData : TPictureData;
    RecivedBitmap : TBitmap32;
    TmpJpeg : TJPEGImage;
begin
  if Assigned(FPictureDataBuffer) and FPictureDataBuffer.TryGetValue(ASenderName, PictureData) then
  begin
    if PictureData.TimeOut > GetTickCount() then
    begin
      TmpJpeg := TJPEGImage.Create();
      try
        APictureStream.Position := 0;
        TmpJpeg.LoadFromStream(APictureStream);
        {$IFDEF DEBUG}
        TMemoryStream(APictureStream).SaveToFile('d:\stream.jpg');
        TmpJpeg.SaveToFile('d:\jpeg.jpg');
        {$ENDIF}
        RecivedBitmap := TBitmap32.Create();
        try
          RecivedBitmap.SetSizeFrom(iDrawArea);
          RecivedBitmap.DrawMode := dmBlend;
          RecivedBitmap.MasterAlpha := $FF;
          RecivedBitmap.Clear(0);
          dmCommonGraphics.DrawGraphicToBitmap(RecivedBitmap, TmpJpeg, PictureData.PictureRect);
          {$IFDEF DEBUG}
          RecivedBitmap.SaveToFile('d:\bitmap.bmp');
          {$ENDIF}
          RecivedBitmap.MasterAlpha := PictureData.MasterAlpha;
          PostRecivedLayer(RecivedBitmap);
        finally
          FreeAndNil(RecivedBitmap);
        end;
      except
      end;
      FreeAndNil(TmpJpeg);
    end;
    FPictureDataBuffer.Remove(ASenderName);
  end;
end;

procedure TfmPaintBoard.DrawToolMouseClick(Sender: TObject);
var i : Integer;
begin
  if FPluginIsExpired then
    Exit;
  if Assigned(FInsertPictureLayer) then  //load picture mode
  begin
    for i := 0 to Self.ComponentCount - 1 do //hiding all panel controls
    begin
      if TControl(Self.Components[i]).Parent = sPTools then
        TControl(Self.Components[i]).Visible := False;
    end;
    sSBPostPicture.Visible := True;
    sSBCancelPicture.Visible := True;
    iDrawArea.Cursor := crDefault;
  end
  else
  begin
    if sSBRect.Down then
    begin
      sSBBorder.ImageIndex := 10;
      sSBBorderAndFill.ImageIndex := 11;
      sSBFill.ImageIndex := 12;
    end;
    if sSBEllipse.Down then
    begin
      sSBBorder.ImageIndex := 3;
      sSBBorderAndFill.ImageIndex := 1;
      sSBFill.ImageIndex := 15;
    end;

    for i := 0 to Self.ComponentCount - 1 do //showing all panel controls
    begin
      if TControl(Self.Components[i]).Parent = sPTools then
        TControl(Self.Components[i]).Visible := True;
    end;

    sSBPostPicture.Visible := False;
    sSBCancelPicture.Visible := False;

    sSBBorder.Visible := sSBRect.Down or sSBEllipse.Down;
    sSBBorderAndFill.Visible := sSBRect.Down or sSBEllipse.Down;
    sSBFill.Visible := sSBRect.Down or sSBEllipse.Down;
    sSBBrushStyle.Visible := sSBBrush.Down;
    sSBStampImage.Visible := sSBStamp.Down;
    sCBFontName.Visible := sSBText.Down;
    sTBFontSize.Visible := sSBText.Down;
    sTBPenSize.Visible := sSBText.Down;
    sSBBold.Visible := sSBText.Down;
    sSBItalic.Visible := sSBText.Down;
    sSBUnderline.Visible := sSBText.Down;
    sTBPenSize.Visible := sSBPen.Down or sSBLine.Down or sSBRect.Down or sSBEllipse.Down or sSBStamp.Down;
    sTBAlphaChanel.Visible := not sSBPicker.Down;
    sCSInticator.Visible := sSBPicker.Down;

    iDrawArea.Cursor := crCross;
    if sSBText.Down then
      begin
        iDrawArea.Cursor := crIBeam;
        FLastSelectedShape := sText;
      end;
    if sSBPicker.Down then
      iDrawArea.Cursor := crPicker;
    if sSBBucket.Down then
      begin
        iDrawArea.Cursor := crBucket;
        FLastSelectedShape := sBucket;
      end;
    if sSBPen.Down then
      begin
        iDrawArea.Cursor := crPencil;
        FLastSelectedShape := sPen;
      end;
    if sSBBrush.Down then
      begin
        iDrawArea.Cursor := crBrush;
        FLastSelectedShape := sPen;
      end;
    if sSBStamp.Down then
      begin
        iDrawArea.Cursor := crStamp;
        FLastSelectedShape := sStamp;
      end;
    if sSBEllipse.Down then
      begin
        iDrawArea.Cursor := crEllipse;
        FLastSelectedShape := sEllipse;
      end;

    if sSBRect.Down then
      begin
        iDrawArea.Cursor := crRectangle;
        FLastSelectedShape := sRect;
      end;
    if sSBLine.Down then
      begin
        iDrawArea.Cursor := crLine;
        FLastSelectedShape := sLine;
      end;
    case FLastSelectedShape of
     sPen  : sTBPenSize.Hint := '<b> </b><br>' + #13#10 +'   .';
     sStamp: sTBPenSize.Hint := '<b> </b><br>' + #13#10 +'    .';
     sRect, sEllipse, sLine: sTBPenSize.Hint := '<b> </b><br>' + #13#10 +'   .';
    end;


    if (not sSBText.Down) and FReadyEnterText then
    begin
      FReadyEnterText := False;
    end;
  end;
  sPTools.Invalidate();
  sPColors.Invalidate();
  SetFocusSafe(iDrawArea);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.AddPointToCurve(X, Y: integer);
var
  APoint: TPoint;
//  ALength : Integer;
begin
  APoint.X := X;
  APoint.Y := Y;
  SetLength(FOrigList, Length(FOrigList) + 1);
  FOrigList[Length(FOrigList) - 1] := APoint;
//  ALength := PolySimplifyArray(0.7, FOrigList);
//  SetLength(FOrigList, ALength);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.Redo(ASendData : Boolean = True);
var ALayer : TBitmapLayer;
    RedoIndex : integer;
begin
  if CanRedo() then
  begin
    ALayer  := (iDrawArea.Layers[0] as TBitmapLayer);
    RedoIndex := lRedo.Bitmaps.Count - 1;
    ALayer.Bitmap := lRedo.Bitmap[RedoIndex]; //this is assign!!!
    lRedo.Bitmaps.Delete(RedoIndex);
    ALayer.Index := (DRAW_LAYER_INDEX - 1);
    Inc(FPostedLayersCount);
    if ASendData then
      SendDataUndo(False);
  end;
end;

procedure TfmPaintBoard.ReDrawTextOnDrawLayer(ADrawCursor : Boolean = True);
begin
  FDrawTimer.RestartTimerText(X0, Y0, FTextToDraw, ADrawCursor);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.OnDrawTimer(Sender: TObject);
var FDT : TDrawTimer;
begin
  FDT := (Sender as TDrawTimer);
  FDT.Enabled := False;
  if True then
  case FDT.FShape of
    sPen, sBrush:
      begin
        if FDT.FFullRedraw then
          FDT.FSegmentsToDrawCount := 0;
        if FDT.FShape = sPen then
          dmCommonGraphics.DrawPolyLine(FDT.FPointArray, DrawLayerBitmap, CurrentPen, FMaskBitmap,
            FDT.FSegmentsToDrawCount, CurrentPenTexture)
        else
          dmCommonGraphics.DrawPolyLine(FDT.FPointArray, DrawLayerBitmap, CurrentBrush, FMaskBitmap,
            FDT.FSegmentsToDrawCount, CurrentPenTexture);
        FDT.FSegmentsToDrawCount := 0;
      end;
    sRect, sEllipse:
      dmCommonGraphics.DrawFigure(FDT.FX0, FDT.FY0, FDT.FX1, FDT.FY1, DrawLayerBitmap,
        FMaskBitmap,   FDT.FShape = sEllipse, FDT.FPenWidth, GetFigureFilling(), CurrentPenTexture, CurrentBrushTexture);

    sText:
      dmCommonGraphics.DrawText(FDT.FX0, FDT.FY0, DrawLayerBitmap, FMaskBitmap, FDT.FText, CurrentPenTexture, FDT.FDrawCursor);

    sLine:
      dmCommonGraphics.DrawLine(FDT.FX0, FDT.FY0, FDT.FX1, FDT.FY1, DrawLayerBitmap, CurrentPen, FMaskBitmap,
        CurrentPenTexture);

    sStamp:
      dmCommonGraphics.DrawStamp(FDT.FX0, FDT.FY0, FDT.FPenWidth * 8, DrawLayerBitmap, CurrentStamp, FMaskBitmap,
        CurrentPenTexture);

    sBucket: ;
  end;
  FDT.FLastExecutedTime := GetTickCount();
end;

procedure TfmPaintBoard.OnKeyPress(Sender: TObject; var Key: Char);
var TxtHeight, TmpY : Integer;
begin
  if not FReadyEnterText then Exit;
  if Ord(Key) = 13 then
    begin
      TxtHeight := DrawLayerBitmap.TextHeight('_|'); //                 
      if Y0 + Round(TxtHeight * 1.5) >= 600 then
        TmpY := Round(TxtHeight * 0.5)
      else
        TmpY := Y0 + Round(TxtHeight * 1.5);
      FReadyEnterText := False;
      ReDrawTextOnDrawLayer(False);
      FDrawTimer.StopEndExecuteIfNeed();
      SendDataText(DrawLayerBitmap.Font, FTextToDraw, X0, Y0);
      PostDrawLayer();
      FTextToDraw := '';
      iDrawAreaMouseDown(iDrawArea, FMouseButtonDrawTextDown, [], X0, TmpY, nil);
    end
  else
    begin
      if Ord(Key) = 27 then
        begin
          FReadyEnterText := False;
          DrawLayerBitmap.Clear(0);
          FTextToDraw := '';
        end
      else
        begin
          if Ord(Key) >= 32 then
            FTextToDraw := FTextToDraw + Key;
          if Ord(Key) = 8 then
            SetLength(FTextToDraw, Length(FTextToDraw) - 1);
          ReDrawTextOnDrawLayer();
        end;
    end;
  Key := Chr(19);
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.OnShowForm(Sender: TObject);
begin
  FixWindowPos();
  iDrawArea.Left := 0;
  iDrawArea.Top := 0;
  iDrawArea.Width := 800;
  iDrawArea.Height := 600;
  DrawToolMouseClick(nil);
  dmCommonGraphics.sSkinManagerPB.UpdateSkin();
  inherited;
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.TextAttrOnChange(Sender: TObject);
begin
  if not FReadyEnterText then Exit;
  With DrawLayerBitmap.Font do
    begin
      Size := sTBFontSize.Position;
      Name := sCBFontName.Text;
      Style := [];
      if sSBBold.Down then
        Style := Style + [fsBold];
      if sSBItalic.Down then
        Style := Style + [fsItalic];
      if sSBUnderline.Down then
        Style := Style + [fsUnderline];
    end;
  if Assigned(Sender) then
    ReDrawTextOnDrawLayer();
end;
procedure TfmPaintBoard.Undo(ASendData : Boolean = True);
var ALayer : TBitmapLayer;
begin
  if CanUndo() then
    begin
      ALayer  := (iDrawArea.Layers[DRAW_LAYER_INDEX - 1] as TBitmapLayer);
      with lRedo.Bitmaps.Add do
        begin
          //this is assign!!!
          Bitmap := ALayer.Bitmap;
        end;
      ALayer.Bitmap.Clear(clBlack);
      ALayer.SendToBack();
      Dec(FPostedLayersCount);
      if ASendData then
        SendDataUndo(True);
    end;
end;

procedure TfmPaintBoard.UpdateTimerTimer(Sender: TObject);
begin
  sSBUndo.Enabled := CanUndo();
  sSBRedo.Enabled := CanRedo();
  sSBLoadPictureFromClipboard.Enabled := Clipboard.HasFormat(CF_BITMAP);
end;

//--------------------------------------------------------------------------------------------------
function TfmPaintBoard.CanRedo: Boolean;
begin
  Result := lRedo.Bitmaps.Count > 0;
end;

function TfmPaintBoard.CanUndo: Boolean;
begin
  Result := (lRedo.Bitmaps.Count < DRAW_LAYER_INDEX - 1) and (FPostedLayersCount > 0);
end;

procedure TfmPaintBoard.ClearRedoStack;
begin
  lRedo.Bitmaps.Clear();
end;

function TfmPaintBoard.CurrentAlphaValue: Integer;
begin
  Result := 255 - sTBAlphaChanel.Position;
end;

function TfmPaintBoard.DecodeDataAsPictureData(ASenderName : String; Data: String): Boolean;
var PictureData : TPictureData;
begin
  if (not IsValidPaintDataString(Data)) or FPluginIsExpired then
    Exit(False);
  try
    if Data[Length(START_PAINT_MESSAGE) + 1] = 'I' then
    begin
      Delete(Data, 1, Length(START_PAINT_MESSAGE) + 1);
      SetLength(Data,  Length(Data) - Length(END_PAINT_MESSAGE));
      PictureData.MasterAlpha := StrToInt('$' + LeftCutStr(Data, 2));
      PictureData.PictureRect.Left := Str32xToInt(LeftCutStr(Data, 2));
      PictureData.PictureRect.Top := Str32xToInt(LeftCutStr(Data, 2));
      PictureData.PictureRect.Right := Str32xToInt(LeftCutStr(Data, 2));
      PictureData.PictureRect.Bottom := Str32xToInt(LeftCutStr(Data, 2));
      PictureData.TimeOut := GetTickCount() + 5000;

      if Assigned(FPictureDataBuffer) then
      begin
        FPictureDataBuffer.AddOrSetValue(ASenderName, PictureData);
      end
      else
      begin
        FPictureDataBuffer := TDictionary<String, TPictureData>.Create();
        FPictureDataBuffer.Add(ASenderName, PictureData);
      end;
      Result := True;
    end
    else
      Result := False;
  except
    on E : Exception do
    begin
      {$IFDEF DEBUG}
      AddLogStr(E.Message, 'Y');
      {$ENDIF}
      Result := False;
    end;
  end;
end;

procedure TfmPaintBoard.DecodeInputString(Data : String);
var PenList : TArrayOfPoint;
    j, i,X1,X2,Y1,Y2 : integer;
    FontStyleInt, PenWidth, BrushStyle, StampStyle : Integer;
    TempData : String;
    StrL : TStringList;
    RecivedShape : Char;
    RecivedBitmap : TBitmap32;
    RecivedPenTexture : TTexture;
    RecivedBrushTexture : TTexture;
    RecivedTemporaryBitmap : TBitmap32;
    RecivedBSP : TMyBitmap32;
    RecivedFigureFilling : TFigureFilling;
begin
  if (not IsValidPaintDataString(Data)) or FPluginIsExpired then
    Exit;
  RecivedBitmap := nil;
  RecivedPenTexture := nil;
  RecivedBrushTexture := nil;
  RecivedTemporaryBitmap := nil;
  RecivedBSP := nil;

  StrL := TStringList.Create();
  try
    StrL.Delimiter := #09;
    StrL.StrictDelimiter := True;
    Delete(Data, 1, Length(START_PAINT_MESSAGE));
    SetLength(Data,  Length(Data) - Length(END_PAINT_MESSAGE));
    StrL.DelimitedText := Data;
    for j := 0 to StrL.Count - 1 do
      begin
        TempData := StrL[j];
        if TempData = 'UNDO' then
          begin
            Undo(False);
            Continue;
          end
        else
        if TempData = 'REDO' then
          begin
            Redo(False);
            Continue;
          end;
        if Length(TempData) < 5 then
          Continue;
        RecivedShape := LeftCutStr(TempData, 1)[1];
        if not CharInSet(RecivedShape, ['F','P','B','R','E','T','C','S']) then
          Continue;
        if not Assigned(RecivedBitmap) then
          begin
            RecivedBitmap := TBitmap32.Create();
            RecivedBitmap.SetSizeFrom(iDrawArea);
            RecivedBitmap.DrawMode := dmBlend;
          end;
        //   
        if TempData = 'LBRD' then
          begin
            RecivedBitmap.MasterAlpha := $FF;
            RecivedBitmap.Clear(clWhite32);
            PostRecivedLayer(RecivedBitmap);
            Continue;
          end;
        if not Assigned(RecivedPenTexture) then
          RecivedPenTexture := TTexture.Create();
        if not Assigned(RecivedTemporaryBitmap) then
          begin
            RecivedTemporaryBitmap := TBitmap32.Create();
            RecivedTemporaryBitmap.SetSizeFrom(iDrawArea);
          end;
        if (not Assigned(RecivedBSP)) and CharInSet(RecivedShape, ['P','B','S']) then
          begin
            RecivedBSP := TMyBitmap32.Create();
          end;


        RecivedBitmap.Clear(0);
        try
          //       
          RecivedBitmap.MasterAlpha := StrToInt('$' + LeftCutStr(TempData, 2));
          RecivedPenTexture.DecodeTexture(LeftCutStr(TempData, 6));
        //          SetLength(PenL

          case RecivedShape of
            'S' :
            begin
              PenWidth := StrToInt('$' + LeftCutStr(TempData, 2));
              StampStyle := StrToInt('$' + LeftCutStr(TempData, 2));
              SetStampImage(RecivedBSP, StampStyle, False);
              X1 := Str32xToInt(LeftCutStr(TempData, 2));
              Y1 := Str32xToInt(LeftCutStr(TempData, 2));
              dmCommonGraphics.DrawStamp(X1, Y1, PenWidth * 8, RecivedBitmap, RecivedBSP, RecivedTemporaryBitmap, RecivedPenTexture);

            end;
            'P', 'B' : //pen or brush or line
            begin
              if RecivedShape = 'P' then
                begin
                  PenWidth := StrToInt('$' + LeftCutStr(TempData, 2));
                  SetPen(RecivedBSP, PenWidth);
                end
              else
                begin
                  BrushStyle := StrToInt('$' + LeftCutStr(TempData, 2));
                  SetBrush(RecivedBSP, BrushStyle, False);
                end;
              SetLength(PenList, Length(TempData) div 4);
              for i := 0 to Length(PenList) - 1 do
                begin
                  PenList[i].X := Str32xToInt(LeftCutStr(TempData, 2));
                  PenList[i].Y := Str32xToInt(LeftCutStr(TempData, 2));
                end;
              dmCommonGraphics.DrawPolyLine(PenList, RecivedBitmap, RecivedBSP, RecivedTemporaryBitmap,
                0, RecivedPenTexture);
            end;
            'R', 'E' : //rectangle or elips
            begin
              if not Assigned(RecivedBrushTexture) then
                RecivedBrushTexture := TTexture.Create();
              RecivedBrushTexture.DecodeTexture(LeftCutStr(TempData, 6));
              RecivedFigureFilling := TFigureFilling(StrToInt(LeftCutStr(TempData, 1)));
              PenWidth := StrToInt('$' + LeftCutStr(TempData, 2));
              X1 := Str32xToInt(LeftCutStr(TempData, 2));
              Y1 := Str32xToInt(LeftCutStr(TempData, 2));
              X2 := Str32xToInt(LeftCutStr(TempData, 2));
              Y2 := Str32xToInt(LeftCutStr(TempData, 2));
              dmCommonGraphics.DrawFigure(X1, Y1, X2, Y2, RecivedBitmap,
                RecivedTemporaryBitmap,   RecivedShape = 'E', PenWidth, RecivedFigureFilling,
                RecivedPenTexture, RecivedBrushTexture);
            end;
            'F' : //bucket (floodfill)
            begin
              X1 := Str32xToInt(LeftCutStr(TempData, 2));
              Y1 := Str32xToInt(LeftCutStr(TempData, 2));
              iDrawArea.Layers[DRAW_LAYER_INDEX].Visible := False;
              try
                dmCommonGraphics.FloodFill(X1, Y1, 20, iDrawArea, RecivedBitmap, RecivedPenTexture);
              finally
                iDrawArea.Layers[DRAW_LAYER_INDEX].Visible := True;
              end;
            end;
            'T' : //text
            begin
              FontStyleInt := StrToInt(LeftCutStr(TempData, 1)[1]);
              with RecivedBitmap.Font do
                begin
                  Style := [];
                  if (FontStyleInt and 1) <> 0 then
                    Style := [fsBold];
                  if (FontStyleInt and 2) <> 0 then
                    Style := Style + [fsItalic];
                  if (FontStyleInt and 4) <> 0 then
                    Style := Style + [fsUnderline];
                end;
              RecivedBitmap.Font.Name := sCBFontName.Items[StrToInt(LeftCutStr(TempData, 1))];
              RecivedBitmap.Font.Size := StrToInt('$' + LeftCutStr(TempData, 2));
              X1 := Str32xToInt(LeftCutStr(TempData, 2));
              Y1 := Str32xToInt(LeftCutStr(TempData, 2));
              TempData := Copy(TempData, 2, Length(TempData) - 2);
              dmCommonGraphics.DrawText(X1, Y1, RecivedBitmap, RecivedTemporaryBitmap, TempData, RecivedPenTexture);
            end;
          end;
          PostRecivedLayer(RecivedBitmap);
        except
          {$IFDEF DEBUG}
          on E : Exception do
            AddLogStr(E.Message, 'Y');
          {$ENDIF}
        end;
      end;
  finally
    FreeAndNilA(StrL);
    FreeAndNilA(RecivedBitmap);
    FreeAndNilA(RecivedPenTexture);
    FreeAndNilA(RecivedBrushTexture);
    FreeAndNilA(RecivedTemporaryBitmap);
    FreeAndNilA(RecivedBSP);
  end;
end;
//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.sSBSaveClick(Sender: TObject);
 var SaveImage32 : TBitmap32;
     SaveImage : TBitmap;
begin
  SaveImage32 := TBitmap32.Create();
  try
    SaveImage32.SetSizeFrom(iDrawArea);
    iDrawArea.PaintTo(SaveImage32, Classes.Rect(0, 0, SaveImage32.Width, SaveImage32.Height));
    if sSavePictureDialog.Execute then
      begin
        SaveImage := TBitmap.Create();
        try
          SaveImage.Assign(SaveImage32);
          try
            SaveImage.SaveToFile(sSavePictureDialog.FileName);
          except
            on E : Exception do
              ShowMessage(E.Message);
          end;
        finally
          FreeAndNilA(SaveImage);
        end;
      end;
  finally
    sSBSave.Down := False;
    FreeAndNilA(SaveImage32);
  end;
end;

procedure TfmPaintBoard.sSBStampImageClick(Sender: TObject);
var TmpInt : Integer;
var SenderButton : TsSpeedButton;
begin
  SenderButton := (Sender as TsSpeedButton);
  SenderButton.Enabled := False;
  try
    TmpInt := ShowSelectorWindow(SenderButton, SenderButton.Images, False, 8, 8);
    if TmpInt <> -1 then
      begin
        SetStampImage(CurrentStamp, TmpInt);
      end;
  finally
    SenderButton.Down := False;
    SenderButton.Enabled := True;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBUndoClick(Sender: TObject);
begin
  (Sender as TsSpeedButton).Down := False;
  Undo();
end;

procedure TfmPaintBoard.sSplitterDblClick(Sender: TObject);
begin
  try
    if Height <= sSplitter.Height then
      ChatWindowHeight := 0
    else
      ChatWindowHeight := 3000;
    MoveWindow(ParentChanHandle, 0, Height, Width, ChatWindowHeight, True);
    BoardResizing := False;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TfmPaintBoard.sSplitterDblClick()');
  end;
end;


procedure TfmPaintBoard.sSplitterMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var PanelSize: TSize;
begin
  try
    StartResizingYCoord := Mouse.CursorPos.Y;
    StartResizingHeight := Height;
    PanelSize := GetParentPanelSize();
    MaxResizingHeigth := PanelSize.cy;
    Width := PanelSize.cx;
    BoardResizing := True;
  except
     on E : Exception do
       AddLogStr(E.Message, 'TfmPaintBoard.sSplitterMouseDown()');
  end;
end;

procedure TfmPaintBoard.sSplitterMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var NewHeight, MinHeight : Integer;
begin
  if BoardResizing then
    begin
      try
        NewHeight := StartResizingHeight + (Mouse.CursorPos.Y - StartResizingYCoord);
        MinHeight := {sPColors.Height + sPTools.Height +} sSplitter.Height;
        if NewHeight < MinHeight then
          NewHeight := MinHeight;
        if NewHeight > MaxResizingHeigth then
          NewHeight := MaxResizingHeigth;
        if Height <> NewHeight then
          begin
            Height := NewHeight;
            ChatWindowHeight := MaxResizingHeigth - Height;
            MoveWindow(ParentChanHandle, 0, Height, Width, ChatWindowHeight, True);
          end;
      except

      end;
    end;
end;

procedure TfmPaintBoard.sSplitterMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  BoardResizing := False;
end;

procedure TfmPaintBoard.sTBAlphaChanelChange(Sender: TObject);
begin
  if Assigned(FInsertPictureLayer) then
  begin
    FInsertPictureLayer.MasterAlpha := CurrentAlphaValue();
  end
  else
  begin
    DrawLayerBitmap.MasterAlpha := CurrentAlphaValue();
    TextAttrOnChange(sTBFontSize);
  end;
end;

procedure TfmPaintBoard.sTBPenSizeChange(Sender: TObject);
begin
  SetPen(FCurrentPen, sTBPenSize.Position);
end;

//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.sSBCopyClick(Sender: TObject);
var MyFormat : Word;
    AData : Cardinal;
    APalette : HPALETTE;
    SaveImage32 : TBitmap32;
    SaveImage : TBitmap;
begin
  SaveImage32 := TBitmap32.Create();
  try
    SaveImage32.SetSizeFrom(iDrawArea);
    iDrawArea.PaintTo(SaveImage32, Classes.Rect(0, 0, SaveImage32.Width, SaveImage32.Height));
    SaveImage := TBitmap.Create();
    try
      SaveImage.Assign(SaveImage32);
      SaveImage.SaveToClipBoardFormat(MyFormat, AData, APalette);
      ClipBoard.SetAsHandle(MyFormat, AData);
    finally
      FreeAndNilA(SaveImage);
    end;
  finally
    sSBCopy.Down := False;
    FreeAndNilA(SaveImage32);
  end;
end;

procedure TfmPaintBoard.sSBLoadPictureFromClipboardClick(Sender: TObject);
var ImageMaxPixelsSize : Integer;
begin
  try
    if Clipboard.HasFormat(CF_BITMAP) then
    begin
      ImageMaxPixelsSize := GetImageLimitation(FParentChanName);
      if (ImageMaxPixelsSize > 0) then
      begin
        FInsertPictureLayer := TPictureLayer.CreatePictureLayer(iDrawArea, '', ImageMaxPixelsSize);
        if Assigned(FInsertPictureLayer) then
        begin
          FInsertPictureLayer.MasterAlpha := CurrentAlphaValue();
          DrawToolMouseClick(nil);
        end
        else
          ShowMessage(BAD_IMAGE_MESSAGE);
      end;
    end;
  finally
    sSBLoadPictureFromClipboard.Down := False;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBLoadPictureFromFileClick(Sender: TObject);
var ImageMaxPixelsSize : Integer;
begin
  try
    ImageMaxPixelsSize := GetImageLimitation(FParentChanName);
    if (ImageMaxPixelsSize > 0) and (sOpenPictureDialog.Execute()) then
    begin
      FInsertPictureLayer := TPictureLayer.CreatePictureLayer(iDrawArea, sOpenPictureDialog.FileName, ImageMaxPixelsSize);
      if Assigned(FInsertPictureLayer) then
      begin
        FInsertPictureLayer.MasterAlpha := CurrentAlphaValue();
        DrawToolMouseClick(nil);
      end
      else
        ShowMessage(BAD_IMAGE_MESSAGE);
    end;
  finally
    sSBLoadPictureFromFile.Down := False;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBPenColorClick(Sender: TObject);
var TmpInt : Integer;
    SenderButton : TsSpeedButton;
    IsPenButtonClick : Boolean;
begin
  SenderButton := (Sender as TsSpeedButton);
  if SenderButton.Tag = 1 then
    Exit;
  SenderButton.Tag := 1;
  SenderButton.Enabled := False;
  try
    IsPenButtonClick := SenderButton = sSBPenColor;

    TmpInt := ShowSelectorWindow(SenderButton, dmCommonGraphics.aiTextures, True, 7, 8);
    case TmpInt of
     -1: ;
     -2:
      begin
        if sColorDialog.Execute then
          SetCurrentTextureOrColor(IsPenButtonClick, -1, sColorDialog.Color);
      end;
    else
      SetCurrentTextureOrColor(IsPenButtonClick, TmpInt);
    end;
  finally
    SenderButton.Down := False;
    SenderButton.Tag := 0;
    SenderButton.Enabled := True;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBPostPictureClick(Sender: TObject);
var TmpStream : TMemoryStream;
    PictureRect : TRect;
begin
  try
    if Assigned(FInsertPictureLayer) then
    begin
      TmpStream := TMemoryStream.Create();
      try
        FInsertPictureLayer.SaveBitmapToStream(PictureRect, TmpStream);
        SendPaintData(FParentChanName, 'I' + IntToHex(FInsertPictureLayer.MasterAlpha, 2) +
                 IntToStr32x(PictureRect.Left) + IntToStr32x(PictureRect.Top) +
                 IntToStr32x(PictureRect.Right) + IntToStr32x(PictureRect.Bottom));
        ImageToChannel(FParentChanName, TmpStream);
        if FIsPrivateChannel then
        begin
          FInsertPictureLayer.DrawBitmap(DrawLayerBitmap);
          PostDrawLayer();
        end;
      finally
        FreeAndNil(TmpStream);
        FreeAndNil(FInsertPictureLayer);
        DrawToolMouseClick(nil);
      end;
    end;
  finally
    sSBPostPicture.Down := False;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBRedoClick(Sender: TObject);
begin
  (Sender as TsSpeedButton).Down := False;
  Redo();
end;

//--------------------------------------------------------------------------------------------------
procedure TfmPaintBoard.pbDrawAreaClick(Sender: TObject);
begin
  if sTBFontSize.Visible then
    SetFocusSafe(sTBFontSize);
end;

procedure TfmPaintBoard.PlaceControls;
var LastLeft, ControlsDistance, ControlsTop : Integer;

  procedure PlaceControl(AControl : TControl);
  begin
    AControl.Left := LastLeft;
    if (AControl is TsSpeedButton) then
    begin
      AControl.Height := 26;
      if TsSpeedButton(AControl).ButtonStyle = tbsButton then
        AControl.Width := 26;
    end
    else
    if (AControl is TsTrackBar) then
    begin
      AControl.Width := 150;
      AControl.Height := 26;
    end;
    AControl.Top := ControlsTop + ((26 - AControl.Height) div 2);
    Inc(LastLeft, AControl.Width + ControlsDistance);
  end;
begin
  Inherited;
  ControlsDistance := 1;
  ControlsTop := 0;
  LastLeft := ControlsDistance;

  PlaceControl(sSBSave);
  if FAllowImageInserting then
    PlaceControl(sSBLoadPictureFromFile)
  else
  begin
    sSBLoadPictureFromFile.Visible := False;
    sSBLoadPictureFromFile.Top := - 100;
  end;
  PlaceControl(sSBCopy);
  if FAllowImageInserting then
    PlaceControl(sSBLoadPictureFromClipboard)
  else
  begin
    sSBLoadPictureFromClipboard.Visible := False;
    sSBLoadPictureFromClipboard.Top := - 100;
  end;

  PlaceControl(sSBClean);
  PlaceControl(sSBUndo);
  PlaceControl(sSBRedo);
  PlaceControl(sSBPen);
  PlaceControl(sSBBrush);
  PlaceControl(sSBStamp);
  PlaceControl(sSBBucket);
  PlaceControl(sSBPicker);
  PlaceControl(sSBRect);
  PlaceControl(sSBEllipse);
  PlaceControl(sSBLine);
  PlaceControl(sSBText);
  //pen related
  PlaceControl(sTBPenSize);

  //FigureRelated
  PlaceControl(sSBBorder);
  PlaceControl(sSBFill);
  PlaceControl(sSBBorderAndFill);

  LastLeft := sTBPenSize.Left;
  //Font Related
  PlaceControl(sTBFontSize);
  PlaceControl(sSBBold);
  PlaceControl(sSBUnderline);
  PlaceControl(sSBItalic);
  PlaceControl(sCBFontName);

  LastLeft := sTBPenSize.Left;
  //brush related
  PlaceControl(sSBBrushStyle);
  LastLeft := sSBBorder.Left;
  //StampRelated
  PlaceControl(sSBStampImage);

  ControlsDistance := 3;
  LastLeft := ControlsDistance;
  //load picture mode
  PlaceControl(sSBPostPicture);
  PlaceControl(sSBCancelPicture);

  ControlsTop := 1;
  ControlsDistance := 2;
  LastLeft := ControlsDistance;
  PlaceControl(sSBPenColor);
  PlaceControl(sSBBrushColor);
  PlaceControl(sColorsPanel);
  PlaceControl(sCSInticator);
  //Alpha Bar
  LastLeft := sCSInticator.Left;
  PlaceControl(sTBAlphaChanel);
end;

procedure TfmPaintBoard.sSBBrushStyleClick(Sender: TObject);
var TmpInt : Integer;
var SenderButton : TsSpeedButton;
begin
  SenderButton := (Sender as TsSpeedButton);
  SenderButton.Enabled := False;
  try
    TmpInt := ShowSelectorWindow(SenderButton, SenderButton.Images, False, 3);
    if TmpInt <> -1 then
      begin
        SetBrush(CurrentBrush, TmpInt);
      end;
  finally
    SenderButton.Down := False;
    SenderButton.Enabled := True;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBCancelPictureClick(Sender: TObject);
begin
  try
    if Assigned(FInsertPictureLayer) then
    begin
      FreeAndNil(FInsertPictureLayer);
      DrawToolMouseClick(nil);
    end;
  finally
    sSBCancelPicture.Down := False;
    SetFocusSafe(iDrawArea);
  end;
end;

procedure TfmPaintBoard.sSBCleanClick(Sender: TObject);
begin
  try
    if MessageBox(Handle, '     ?', 'O PaintBoard', MB_ICONQUESTION + MB_YESNO + MB_DEFBUTTON2) = IDYES then
    begin
      DrawLayerBitmap.MasterAlpha := $FF;
      DrawLayerBitmap.Clear(clWhite32);
      PostDrawLayer();
      SendDataClear();
    end;
  finally
    sSBClean.Down := False;
    SetFocusSafe(iDrawArea);
  end;
end;

{ TDrawTimer }

constructor TDrawTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Enabled := False;
  FLastExecutedTime := 0;
  FSegmentsToDrawCount := 0;
  Interval := 20;
end;

procedure TDrawTimer.RestartIfNeed;
begin
  if (not Self.Enabled) or (GetTickCount() < (FLastExecutedTime + 10)) then
    begin
      Self.Enabled := False;
      Self.Enabled := True;
    end;
end;

procedure TDrawTimer.RestartTimer(AX0, AY0, AX1, AY1, APenWidth : Integer; AShape : TShape);
begin
  FX0 := AX0;
  FY0 := AY0;
  FX1 := AX1;
  FY1 := AY1;
  FShape := AShape;
  FPenWidth := APenWidth;
  RestartIfNeed();
end;

procedure TDrawTimer.RestartTimerPenOrBrush(APointArray: TArrayOfPoint; AShape : TShape; AFullRedraw : Boolean = False);
begin
  FSegmentsToDrawCount := FSegmentsToDrawCount + 1;
  FPointArray := APointArray;
  FFullRedraw := AFullRedraw;
  FShape := AShape;
  RestartIfNeed();
end;

procedure TDrawTimer.RestartTimerText(AX0, AY0 : Integer; AText: String;
  ADrawCursor: Boolean);
begin
  FX0 := AX0;
  FY0 := AY0;
  FShape := sText;
  FText := AText;
  FDrawCursor := ADrawCursor;
  RestartIfNeed();
end;

procedure TDrawTimer.StopEndExecuteIfNeed;
begin
  if Self.Enabled then
    begin
      Self.Enabled := False;
      Self.OnTimer(Self);
    end;
end;

initialization

end.
