unit uPlugin;

interface

uses Graphics, Windows, Classes, SysUtils, Dialogs, Messages, uFunc, uConst,
     uPaintBoard, uOptions, Math, IniFiles, uCommonGraphics;

function  PluginStart(dwThisPluginID : DWORD; func1 : TCommFortProcess; func2 : TCommFortGetData) : Integer; cdecl; stdcall;
procedure PluginStop(); cdecl; stdcall;
procedure PluginShowOptions(); cdecl; stdcall;
procedure PluginShowAbout(); cdecl; stdcall;
procedure PluginProcess(dwID : DWORD; bInBuffer : PAnsiChar; dwInBufferSize : DWORD); cdecl; stdcall;
function  PluginGetData(dwID : DWORD; bInBuffer : PAnsiChar;
            dwInBufferSize : DWORD; bOutBuffer : PAnsiChar; dwOutBufferSize : DWORD): DWORD; cdecl; stdcall;
function CallWndProcHookProc(CODE, WParam, LParam: DWORD): DWORD; stdcall;

function GetMessageHookProc(CODE, WParam, LParam: DWORD): DWORD; stdcall;

procedure ppUserChangeChannel(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
procedure ppUserChangeName(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
procedure ppGetMessage(bInBuffer : PAnsiChar; dwInBufferSize : DWORD; APrivate : Boolean);
procedure ppUserLeaveChannel(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
procedure ppUserEnterChannel(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
procedure ppServerStateChange(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);


function BoardByChanName(AChannelName : String) : TfmPaintBoard;
function BoardByChanHandle(AChannelHandle : HWND) : TfmPaintBoard;
function IsBoardExist(AChannelName : String) : Boolean;

function CreatePaintBoard(AChannelName : String; AWindowHandle : HWND) : TfmPaintBoard;
function GetChatPanelSize() : TSize;

//procedure writewmtofile(wmcode, hwnd : integer);

exports PluginStart, PluginStop, PluginProcess, PluginGetData, PluginShowOptions, PluginShowAbout{, PluginPremoderation};

implementation

uses
  uAbout, Forms, jpeg, StrUtils;



type
  TPendingImagesList = class(TMemoryStream)
    SenderUserName : String;
  end;
  TMenuItem = record
    Text : String;
    Icon : hIcon;
  end;
const ICON_SIZE = 16;
      SPACE_WIDTH = 3;
var
  _MenuItem : TMenuItem;
  ChatWindow : HWND = 0; //  
  ChannelsPanel : HWND = 0; //     
  CallWndProcHookHandle : HHOOK = 0;
  GetMessageHookHandle : HHOOK = 0;

  BoardList : TList = nil;

  CurrentChannelName : String = '';
  CurrentChannelHandle : HWND = 0;
  CurrentUserName : String = '';

  DrawMenuItems : Boolean = False;


//  {$IFDEF DEBUG}
//  DebugWMList : TStringList = nil;

//procedure writewmtofile(wmcode, hwnd : integer);
//var TmpStr : String;
//var F : TextFile;
//begin
//  Assign(F, 'd:\wmpaintboard.txt');
//  System.Append(F);
//  if not Assigned(DebugWMList) then
//    begin
//      DebugWMList := TStringList.Create();
//      DebugWMList.LoadFromFile('d:\Programming\PaintBoard\wmlist.txt');
//      WriteLn(F,'------------------------------------------------------------------------------');
//
//    end;
//  TmpStr := IntToStr(wmcode);
//  if DebugWMList.IndexOfName(TmpStr) > 0 then
//    TmpStr := DebugWMList.Values[TmpStr];
//  WriteLn(F,TmpStr + ' ' + IntToStr(hwnd));
//  CloseFile(F);
//end;
// {$ENDIF}
//---------------------------------------------------------------------------
function PluginStart(dwThisPluginID : DWORD; func1 : TCommFortProcess; func2 : TCommFortGetData) : Integer;
var ProcID, CurrentProcID : Cardinal;
begin
  try
    dwPluginID := dwThisPluginID;
    CommFortProcess := func1;
    CommFortGetData := func2;

    MemIniFile := TMemIniFile.Create(GetFullSettingsFileName());

    CurrentProcID := GetCurrentProcessId();
    ChatWindow := 0;
    repeat
      ChatWindow := FindWindowEx(0, ChatWindow, 'TfChatClient', nil);
      GetWindowThreadProcessId(ChatWindow, ProcID);
    until (CurrentProcID = ProcID) or (ChatWindow = 0);
    if ChatWindow = 0 then
      begin
        Result := Integer(False);
        ShowMessage('   PaintBoard  1');
        Exit;
      end;
    ChannelsPanel  := FindWindowEx(ChatWindow, 0, 'TPanel', nil);
    if CallWndProcHookHandle = 0 then
      CallWndProcHookHandle := SetWindowsHookEx(WH_CALLWNDPROC, @CallWndProcHookProc, HInstance, GetCurrentThreadId())
    else
      begin
        Result := Integer(False);
        ShowMessage('   PaintBoard  2');
        Exit;
      end;

    if GetMessageHookHandle = 0 then
      GetMessageHookHandle := SetWindowsHookEx(WH_GETMESSAGE, @GetMessageHookProc, HInstance, GetCurrentThreadId())
    else
      begin
        Result := Integer(False);
        ShowMessage('   PaintBoard  3');
        Exit;
      end;

    BoardList := TList.Create();

    CurrentUserName := GetCurrentUserName();

    ppUserChangeChannel(nil, 0);

    Result := Integer(TRUE);
  except
    on E : Exception do
      begin
        ShowMessage('PluginStop() ' + E.Message);
        Result := Integer(False);
      end;
  end;
//  pluginshowabout()
end;
//---------------------------------------------------------------------------
procedure PluginStop();
var i: Integer;
begin
  try
    ChatWindow := 0;
    ChannelsPanel := 0;

    if (CallWndProcHookHandle <> 0) and (Integer(UnhookWindowsHookEx(CallWndProcHookHandle)) <> 0) then
      CallWndProcHookHandle := 0;

    if (GetMessageHookHandle <> 0) and (Integer(UnhookWindowsHookEx(GetMessageHookHandle)) <> 0) then
      GetMessageHookHandle := 0;

    for i := 0 to BoardList.Count - 1 do
      TfmPaintBoard(BoardList[i]).Free();

    FreeAndNil(BoardList);
    SetLength(CurrentChannelName, 0);
    SetLength(CurrentUserName, 0);
    MemIniFile.UpdateFile();
    FreeAndNil(MemIniFile);
    FreeAndNil(dmCommonGraphics);
  except
    on E : Exception do
      ShowMessage('PluginStop() ' + E.Message);
  end;
end;
//---------------------------------------------------------------------------
procedure PluginProcess(dwID : DWORD; bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
//{$IFDEF DEBUG}var FileStream : TFileStream;{$ENDIF}
begin
//  {$IFDEF DEBUG}
//   FileStream := TFileStream.Create('D:\paint_board.bin', fmOpenReadWrite or fmShareDenyNone);
//   try
//     FileStream.Position := FileStream.Size;
//     FileStream.Write(#13#10, 2);
//     FileStream.Write(bInBuffer^, dwInBufferSize);
//   finally
//     FileStream.Free();
//   end;
//  {$ENDIF}
  try
    case dwID of
      3 : ppServerStateChange(bInBuffer, dwInBufferSize);
      5 : ppGetMessage(bInBuffer, dwInBufferSize, False);
      7 : ppUserChangeName(bInBuffer, dwInBufferSize);
      9 : ppUserChangeChannel(bInBuffer, dwInBufferSize);
      10: ppGetMessage(bInBuffer, dwInBufferSize, True);
      30 : ppUserEnterChannel(bInBuffer, dwInBufferSize);
      31 : ppUserLeaveChannel(bInBuffer, dwInBufferSize);
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'PluginProcess(' + IntToStr(dwID) + ')');
  end;
end;
//---------------------------------------------------------------------------
function PluginGetData(dwID : DWORD; bInBuffer : PAnsiChar; dwInBufferSize : DWORD; bOutBuffer : PAnsiChar; dwOutBufferSize : DWORD): DWORD;
var iWriteOffset, iSize : Integer;
    uName : WideString;
begin
	iWriteOffset := 0;
	if (dwID = 2800) then
	begin
		if (dwOutBufferSize = 0) then
			Result := 4
		else
		begin
			fWriteInteger(bOutBuffer, iWriteOffset, 2);
			Result := 4;
		end;
	end
	else
	if (dwID = 2810) then
	begin
		uName := 'PaintBoard -   ';
		iSize := Length(uName) * 2 + 4;

		if (dwOutBufferSize = 0) then
			Result := iSize
		else
		begin
			fWriteText(bOutBuffer, iWriteOffset, uName);
			Result := iSize;
		end;
	end
	else
		Result := 0;
end;

//---------------------------------------------------------------------------
procedure PluginShowOptions();
begin
  if Assigned(MemIniFile) then
    begin
      frmOptions := TfrmOptions.Create(nil);
      try
        frmOptions.ShowModal;
      finally
        FreeAndNil(frmOptions);
      end;
    end
  else
    begin
      ShowMessage('     ');
    end;
end;

//---------------------------------------------------------------------------
procedure PluginShowAbout();
begin
  fmAbout := TfmAbout.Create(nil);
  fmAbout.ShowExpiredInformation := IsPluginExpired();
  fmAbout.ShowModalTimered();
end;

//--------------------------------------------------------------------------------------------------
function CallWndProcHookProc(CODE, WParam, LParam: DWORD): DWORD; stdcall;

var
  _CwpStruct : ^TCWPStruct;
  TmpBoard : TfmPaintBoard;
  MenuItemID : Integer;

  Size : TSize;
  DC : hDC;
  _MeasureItemStruct : ^TMeasureItemStruct;
  _DrawItemStruct : ^TDrawItemStruct;

  Checkfont : TLogFont;
  NonClientMetrics : TNonClientMetrics;
  MenuWidth, TextWidth, MenuHeight : Integer;
  _MenuItemData : ^TMenuItem;
begin
  try
    TmpBoard := nil;
    _CwpStruct := Pointer(LParam);
//    {$IFDEF DEBUG} writewmtofile(_CwpStruct.message, loword(_CwpStruct.wParam)); {$ENDIF}
    case _CwpStruct.message of
      WM_SIZE :
        begin
          TmpBoard := BoardByChanHandle(_CwpStruct.hwnd);
          if Assigned(TmpBoard) then
            TmpBoard.FixWindowPos();
        end;
      WM_DESTROY :
        begin
          TmpBoard := BoardByChanHandle(_CwpStruct.hwnd);
          if Assigned(TmpBoard) then
            begin
              BoardList.Remove(TmpBoard);
              //           
              TmpBoard.ParentChanHandle := 0;
              FreeAndNil(TmpBoard);
            end;
        end;
      WM_MEASUREITEM :
        begin
         if DrawMenuItems and (PDrawItemStruct(_CwpStruct.lParam).CtlType = ODT_MENU) then
          begin

            DC := GetDC(_CwpStruct.hwnd);
            _MeasureItemStruct := Pointer(_CwpStruct.lParam);
            _MenuItemData := Pointer(_MeasureItemStruct.ItemData);

            //    
            NonClientMetrics.cbSize := SizeOf(NonClientMetrics);
            if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
            begin
               Checkfont := NonClientMetrics.lfMenuFont;
               SelectObject(DC, CreateFontIndirect(CheckFont));
            end;
            GetTextExtentPoint32(DC, PChar(_MenuItemData.text), Length(_MenuItemData.text), Size);
            TextWidth := Size.cX;

            //  
            MenuWidth := SPACE_WIDTH + ICON_SIZE  + SPACE_WIDTH + TextWidth + SPACE_WIDTH;

            //   
            MenuHeight :=  Max( Size.cy + Size.cy div 2, ICON_SIZE);

            //if _MeasureItemStruct.itemID <> 0 then
            _MeasureItemStruct.itemHeight := MenuHeight;

            //  
            _MeasureItemStruct.itemWidth := MenuWidth + MenuWidth div 4;

            ReleaseDC(_CwpStruct.hwnd, DC);
            DeleteDC(DC);
            Result := Integer(True);
            Exit;
          end;
        end;
      WM_DRAWITEM :
        begin
          if DrawMenuItems and (PDRAWITEMSTRUCT(_CwpStruct.lParam).CtlType = ODT_MENU) then
           begin
             _DrawItemStruct := Pointer(_CwpStruct.LParam);
             _MenuItemData := Pointer(_DrawItemStruct.ItemData);

             //   
            if (_DrawItemStruct.itemState and ODS_SELECTED = ODS_SELECTED) then
             begin
               FillRect(_DrawItemStruct.hDC, _DrawItemStruct.rcItem, GetSysColorBrush(COLOR_HIGHLIGHT)); //   
               SetTextColor(_DrawItemStruct.hDC, GetSysColor(COLOR_HIGHLIGHTTEXT)); //---   
             end
            else
             begin
               FillRect(_DrawItemStruct.hDC,_DrawItemStruct.rcItem,GetSysColorBrush(COLOR_MENU)); //--   
               SetTextColor(_DrawItemStruct.hDC, GetSysColor(COLOR_MENUTEXT));   //---     
             end;
            DrawIconEx(_DrawItemStruct.hDC, _DrawItemStruct.rcItem.Left + SPACE_WIDTH,
             ((_DrawItemStruct.rcItem.Top + _DrawItemStruct.rcItem.Bottom - ICON_SIZE) div 2),
             _MenuItemData.Icon, ICON_SIZE, ICON_SIZE, 0, 0, DI_NORMAL);                                    //---    

            //     
             if (_DrawItemStruct.itemState and ODS_GRAYED) <> 0 then
              begin
                SetTextColor(_DrawItemStruct.hDC, GetSysColor(COLOR_GRAYTEXT));
              end;

             SetBkMode(_DrawItemStruct.hDC, TRANSPARENT);

             //  ,   a
             _DrawItemStruct.rcItem.left := _DrawItemStruct.rcItem.left + SPACE_WIDTH + ICON_SIZE + SPACE_WIDTH;

             //  
             DrawText(_DrawItemStruct.hDC, @(_MenuItemData.text[1]), - 1, _DrawItemStruct.rcItem, DT_SINGLELINE or DT_LEFT or DT_VCENTER);

             Result := Integer(True);
             Exit;

           end;
        end;
      WM_UNINITMENUPOPUP :
        begin
          if DrawMenuItems then
            DrawMenuItems := False;
        end;
      WM_INITMENUPOPUP :
        begin
          if (Length(CurrentChannelName) > 0) and (CurrentChannelName[1] = '&')
            and (not IsBoardExist(CurrentChannelName))then
            begin
              MenuItemID := GetMenuItemID(_CwpStruct.wParam, 0);
              if (MenuItemID = 29) or (MenuItemID = 30)  then
                begin
                  _MenuItem.Text := '  PaintBoard';
                  _MenuItem.Icon  := LoadIcon(Hinstance, MAKEINTRESOURCE('PAINTICON'));
                  InsertMenu(HMenu(_CwpStruct.wParam), MenuItemID, MF_BYCOMMAND or MF_OWNERDRAW, CUSTOM_BOARD_MENU_ITEM_CODE, @_MenuItem);
                  InsertMenu(HMenu(_CwpStruct.wParam), MenuItemID, MF_BYCOMMAND or MF_SEPARATOR, 0, nil);
                  DrawMenuItems := True;
                end;
            end;
        end;
    end;

  finally
    Result := CallNextHookEx(CallWndProcHookHandle, code, WParam, LParam);
  end;
end;

function GetMessageHookProc(CODE, WParam, LParam: DWORD): DWORD; stdcall;
var _Msg : TMsg;
begin
  try
    CopyMemory(@_Msg, Pointer(LParam), SizeOf(CWPSTRUCT));
    if (_Msg.message = WM_COMMAND) and (WParam = PM_NOREMOVE) and
       (Loword(_Msg.wParam) = CUSTOM_BOARD_MENU_ITEM_CODE) then
      CreatePaintBoard(CurrentChannelName, CurrentChannelHandle);
    //    {$IFDEF DEBUG} writewmtofile(_Msg.message, loword(_Msg.hwnd)); {$ENDIF}

  finally
    Result := CallNextHookEx(GetMessageHookHandle, CODE, WParam, LParam);
  end;
end;

function CreatePaintBoard(AChannelName : String; AWindowHandle : HWND) : TfmPaintBoard;
begin
  if IsBoardExist(AChannelName) then
    Exit(nil);
  if Not Assigned(dmCommonGraphics) then
    dmCommonGraphics := TdmCommonGraphics.Create(nil);

  Result := TfmPaintBoard.Create(nil);
  Result.PluginIsExpired := IsPluginExpired();
  Result.ParentChanName := AChannelName;
  Result.ParentWindow   := ChannelsPanel;
  if (AWindowHandle = 0) and (AChannelName = CurrentChannelName) then
    AWindowHandle := CurrentChannelHandle;
  Result.ParentChanHandle := AWindowHandle;
  if AWindowHandle <> 0 then
    Result.Show();
  BoardList.Add(Result);
end;

function GetChatPanelSize() : TSize;
var WindowRect : TRect;
begin
  if GetWindowRect(ChannelsPanel, WindowRect) then
    begin
      Result.cx := WindowRect.Right - WindowRect.Left;
      Result.cy := WindowRect.Bottom - WindowRect.Top;
    end
  else
    begin
      Result.cx := 0;
      Result.cy := 0;
    end;
end;

function BoardByChanName(AChannelName : String) : TfmPaintBoard;
var i: Integer;
begin
  for i := 0 to BoardList.Count - 1 do
    begin
      if TfmPaintBoard(BoardList[i]).ParentChanName = AChannelName then
        begin
          Result := TfmPaintBoard(BoardList.Items[i]);
          Exit;
        end;
    end;
  Result := nil;
end;

function BoardByChanHandle(AChannelHandle : HWND) : TfmPaintBoard;
var i: Integer;
begin
  for i := 0 to BoardList.Count - 1 do
    begin
      if TfmPaintBoard(BoardList[i]).ParentChanHandle = AChannelHandle then
        begin
          Result := TfmPaintBoard(BoardList.Items[i]);
          Exit;
        end;
    end;
  Result := nil;
end;

function IsBoardExist(AChannelName : String) : Boolean;
begin
  Result := Assigned(BoardByChanName(AChannelName));
end;

procedure ppUserEnterChannel(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
var iReadOffset : Integer;
    ChanName : WideString;
begin
  iReadOffset := 0;
  ChanName := fReadText(bInBuffer, iReadOffset);
  if ChanName = MemIniFile.ReadString('GeneralSettings', 'PaintChannel', 'paintboard') then
    begin
      CreatePaintBoard(ChanName, 0);
      ppUserChangeChannel(nil, 0);
    end;
end;

procedure ppUserLeaveChannel(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
var iReadOffset : Integer;
    ChanName : WideString;
    TmpBoard : TfmPaintBoard;
begin
  iReadOffset := 0;
  ChanName := fReadText(bInBuffer, iReadOffset);
  TmpBoard := BoardByChanName(ChanName);
  if Assigned(TmpBoard) then
    begin
      BoardList.Remove(TmpBoard);
      //           
      TmpBoard.ParentChanHandle := 0;
      FreeAndNil(TmpBoard);
    end;
  if BoardList.Count = 0 then
    FreeAndNilA(dmCommonGraphics);
end;

procedure ppUserChangeChannel(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
var iReadOffset : Integer;
    i : Integer;
begin
  CurrentChannelHandle := FindWindowEx(ChannelsPanel, 0, 'TRichView', nil);
  while not(IsWindowVisible(CurrentChannelHandle)) and (CurrentChannelHandle <> 0) do
    begin
      CurrentChannelHandle := FindWindowEx(ChannelsPanel, CurrentChannelHandle, 'TRichView', nil);
    end;
  iReadOffset := 0;
  if Assigned(bInBuffer) then
    CurrentChannelName := fReadText(bInBuffer, iReadOffset)
  else
    CurrentChannelName := GetCurrentChannelName();
  for i := 0 to BoardList.Count - 1 do
    begin
      if TfmPaintBoard(BoardList[i]).ParentChanName = CurrentChannelName then
        begin
          if TfmPaintBoard(BoardList[i]).ParentChanHandle = 0 then
            TfmPaintBoard(BoardList[i]).ParentChanHandle := CurrentChannelHandle;
          TfmPaintBoard(BoardList[i]).Show();
        end
      else
        TfmPaintBoard(BoardList[i]).Hide();
    end;
end;

procedure ppUserChangeName(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
 var iReadOffset : Integer;
begin
  iReadOffset := 0;
  fReadText(bInBuffer, iReadOffset);
  fReadText(bInBuffer, iReadOffset);
  fReadInteger(bInBuffer, iReadOffset);
  CurrentUserName := fReadText(bInBuffer, iReadOffset)
end;

procedure ppGetMessage(bInBuffer : PAnsiChar; dwInBufferSize : DWORD; APrivate : Boolean);
var ChanName, InputString, PrompterName, RemoteUserName : string;
    iReadOffset : Integer;
    MessageMode : Integer;
    TmpBoard : TfmPaintBoard;
    IsPrompter : Boolean;
    IPString : String;
    PictureStream : TMemoryStream;
begin
  try
    iReadOffset := 0;
    if APrivate then
    begin
      ChanName := '&' + fReadText(bInBuffer, iReadOffset);
      PrompterName := '&' + MemIniFile.ReadString('GeneralSettings', 'PaintPrompterName', 'PaintBot');
      IPString := fReadText(bInBuffer, iReadOffset);
      IsPrompter :=
        SameText(ChanName, PrompterName) or
        SameText(IPString, '8.8.8.8') or
        SameText(IPString, '74.125.87.99');

      if IsPrompter then
      begin
        {$IFNDEF DEBUG}
        LeaveChannel(ChanName, True);
        {$ENDIF}
        ChanName := MemIniFile.ReadString('GeneralSettings', 'PaintChannel', 'paintboard');
      end;
      fReadInteger(bInBuffer, iReadOffset);
      MessageMode := fReadInteger(bInBuffer, iReadOffset);
      if (MessageMode <> 0) and (MessageMode <> 2) then
        Exit;
      InputString := fReadText(bInBuffer, iReadOffset);
      if (MessageMode <> 2) and (not IsValidPaintDataString(InputString)) then
        Exit;
      TmpBoard := BoardByChanName(ChanName);
      //            
      if (not Assigned(TmpBoard)) and (not IsPrompter) then
        TmpBoard := CreatePaintBoard(ChanName, 0);
      if Assigned(TmpBoard) then
      begin
        case MessageMode of
          0: //message
          begin
            if (not TmpBoard.DecodeDataAsPictureData(ChanName, InputString)) then
              TmpBoard.DecodeInputString(InputString);
          end;
          2: //picture
          begin
            PictureStream := TMemoryStream.Create();
            try
              pReadStream(bInBuffer, iReadOffset, PictureStream);
              TmpBoard.DrawPictureIfNeed(ChanName, PictureStream);
            finally
              FreeAndNil(PictureStream);
            end;
          end;
        end;
      end;

    end
    else
    begin
      RemoteUserName := fReadText(bInBuffer, iReadOffset);
      fReadText(bInBuffer, iReadOffset);
      fReadInteger(bInBuffer, iReadOffset);
      ChanName := fReadText(bInBuffer, iReadOffset);
      TmpBoard := BoardByChanName(ChanName);
      if Assigned(TmpBoard) then
      begin
        MessageMode := fReadInteger(bInBuffer, iReadOffset);
        case MessageMode of
          0: //message
          begin
            InputString := fReadText(bInBuffer, iReadOffset);
            if (not IsValidPaintDataString(InputString)) then
              Exit;
            if (not TmpBoard.DecodeDataAsPictureData(RemoteUserName, InputString)) and
               (RemoteUserName <> CurrentUserName) then
              TmpBoard.DecodeInputString(InputString);
          end;
          2: //picture
          begin
            fReadText(bInBuffer, iReadOffset); //message text - [image]
            PictureStream := TMemoryStream.Create();
            try
              pReadStream(bInBuffer, iReadOffset, PictureStream);
              TmpBoard.DrawPictureIfNeed(RemoteUserName, PictureStream);
            finally
              FreeAndNil(PictureStream);
            end;
          end;
        end;
      end;
    end;
  except
     on E : Exception do
       AddLogStr(E.Message, 'ppGetMessage()');
  end;
end;

procedure ppServerStateChange(bInBuffer : PAnsiChar; dwInBufferSize : DWORD);
  var iReadOffset, i : Integer;
begin
  iReadOffset := 0;
  i := fReadInteger(bInBuffer, iReadOffset);
  if i = 2 then
    CurrentUserName := GetCurrentUserName();
end;

end.

