 unit uPopUpWindow;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, sBitBtn,
  ImgList, sScrollBox, Math;

type
  TfPopup = class(TForm)
    sbContainer: TsScrollBox;
  private
    { Private declarations }
  public
    SelectedButtonIndex : Integer;
    procedure OnButtonClick(Sender: TObject);
    { Public declarations }
  end;

function ShowSelectorWindow(AParentControl : TControl; AImageList : TCustomImageList; AShowChangeColorButton : Boolean = False;
  AMaxColumnsCount : Integer = 6; AMaxRowsCount : Integer = 6) : Integer;


implementation

{$R *.dfm}

function ShowSelectorWindow(AParentControl : TControl; AImageList : TCustomImageList; AShowChangeColorButton : Boolean = False;
  AMaxColumnsCount : Integer = 6; AMaxRowsCount : Integer = 6) : Integer;
const ScrollBarWidth = 16;
      SpaceBetwenButtons = 3;
      AdditionalSizeToButton = 3;
      ChangeCollorButtonHeight = 20;
var fPopup : TfPopup;
    FPoint : TPoint;
    XPos, YPos : Integer;

    i, j : Integer;
    ButtonWidth, ButtonHeight : Integer;
    RowCount, ColumnCount, ImageIndexCounter : Integer;
    TmpButton : TsBitBtn;
    ButtonsList : TList;
    ACCButtonAdditionalWidth : Integer;
begin
  fPopup := TfPopup.Create(nil);
  try
    //Set Size
    ButtonWidth := AImageList.Width + AdditionalSizeToButton * 2;
    ButtonHeight := AImageList.Height + AdditionalSizeToButton * 2;
    ColumnCount := Min(AImageList.Count, AMaxColumnsCount);
    fPopup.ClientWidth := (ButtonWidth + SpaceBetwenButtons) * ColumnCount
      + SpaceBetwenButtons + fPopup.ClientWidth - fPopup.sbContainer.ClientWidth;
    RowCount := Ceil(AImageList.Count / ColumnCount);

    if AShowChangeColorButton then
      ACCButtonAdditionalWidth := AdditionalSizeToButton + ChangeCollorButtonHeight
    else
      ACCButtonAdditionalWidth := 0;

    fPopup.ClientHeight := ((ButtonHeight + SpaceBetwenButtons) *
      Min(RowCount, AMaxRowsCount)) + SpaceBetwenButtons + fPopup.ClientHeight -
      fPopup.sbContainer.ClientHeight + ACCButtonAdditionalWidth;


    if RowCount > AMaxRowsCount then
      begin
        fPopup.Width := fPopup.Width + ScrollBarWidth;
      end;

    //Set Position
    FPoint := AParentControl.ClientToScreen(Point(0,0));
    XPos := FPoint.X;
    if (XPos + fPopup.Width) > Screen.Width then
      XPos := FPoint.X + AParentControl.Width - fPopup.Width;
    if AShowChangeColorButton then
      begin
        YPos := FPoint.Y - fPopup.Height;
        if YPos < 0 then
          YPos := FPoint.Y + AParentControl.Height;
      end
    else
      begin
        YPos := FPoint.Y + AParentControl.Height;
        if (YPos + fPopup.Height) > Screen.Height then
          YPos := FPoint.Y - fPopup.Height;
      end;
    fPopup.Left := XPos;
    fPopup.Top := YPos;
    fPopup.SelectedButtonIndex := -1;

    //Create Buttons
    ButtonsList := TList.Create();
    try
      ImageIndexCounter := 0;
      if AShowChangeColorButton then
        begin
          TmpButton := TsBitBtn.Create(fPopup.sbContainer);
          TmpButton.Parent := fPopup.sbContainer;
          ButtonsList.Add(TmpButton);
          TmpButton.Width  := ((ButtonWidth + SpaceBetwenButtons) * ColumnCount) - SpaceBetwenButtons;
          TmpButton.Height := ChangeCollorButtonHeight;
          TmpButton.Left := SpaceBetwenButtons;
          TmpButton.Top  := SpaceBetwenButtons;
          TmpButton.Caption := ' ';
          TmpButton.Tag := -2;
          TmpButton.OnClick := fPopup.OnButtonClick;
        end;
      for j := 0 to RowCount - 1 do
        begin
          for i := 0 to ColumnCount - 1 do
            begin
              TmpButton := TsBitBtn.Create(fPopup.sbContainer);
              TmpButton.Parent := fPopup.sbContainer;
              ButtonsList.Add(TmpButton);
              TmpButton.Width  := ButtonWidth;
              TmpButton.Height := ButtonHeight;
              TmpButton.Left := SpaceBetwenButtons + (i * (ButtonWidth  + SpaceBetwenButtons));
              TmpButton.Top  := ACCButtonAdditionalWidth + SpaceBetwenButtons + (j * (ButtonHeight + SpaceBetwenButtons));
              TmpButton.Caption := '';//IntToStr(ImageIndexCounter);
              TmpButton.Images := AImageList;
              TmpButton.ImageIndex := ImageIndexCounter;
              TmpButton.Tag := ImageIndexCounter;
              TmpButton.OnClick := fPopup.OnButtonClick;
              Inc(ImageIndexCounter);
              if ImageIndexCounter >= AImageList.Count then
                Break;
            end;
          if ImageIndexCounter >= AImageList.Count then
            Break;
        end;

      //Show until lost focus or close
      fPopup.Show;
      Screen.FocusedForm := fPopup;
      while fPopup.Visible and fPopup.Active do
        Application.ProcessMessages();
      Result := fPopup.SelectedButtonIndex;
    finally
      for i := 0 to ButtonsList.Count - 1 do
        TObject(ButtonsList[i]).Free();
      ButtonsList.Free();
    end;
  finally
    fPopup.Free();
  end;
end;
{ TfPopup }

procedure TfPopup.OnButtonClick(Sender: TObject);
begin
  SelectedButtonIndex := (Sender as TControl).Tag;
  Self.Close();
end;

end.
